@@ -229,6 +229,27 @@ gboolean
   return _gdk_color_state_equal (self, other);
 }
 
+/**
+ * gdk_color_state_equivalent:
+ * @self: a `GdkColorState`
+ * @other: another `GdkColorStatee`
+ *
+ * Compares two `GdkColorStates` for equivalence.
+ *
+ * Two objects that represent the same color state should be equivalent,
+ * even though they may not be equal in the sense of [method@Gdk.ColorState.equal].
+ *
+ * Returns: %TRUE if the two color states are equivalent
+ *
+ * Since: 4.20
+ */
+gboolean
+(gdk_color_state_equivalent) (GdkColorState *self,
+                              GdkColorState *other)
+{
+  return _gdk_color_state_equivalent (self, other);
+}
+
 /**
  * gdk_color_state_create_cicp_params:
  * @self: a `GdkColorState`
@@ -886,6 +907,9 @@ transform_to_cicp (GdkCicpColorState *self,
           values[i][1] = res[1];
           values[i][2] = res[2] + 0.5;
         }
+      values[i][0] = CLAMP (values[i][0], 0.0, 1.0);
+      values[i][1] = CLAMP (values[i][1], 0.0, 1.0);
+      values[i][2] = CLAMP (values[i][2], 0.0, 1.0);
       if (self->cicp.range == GDK_CICP_RANGE_NARROW)
         {
           values[i][0] = values[i][0] * 224.0 / 255.0 + 16.0 / 255.0;
@@ -1276,7 +1300,7 @@ gdk_color_state_yuv (void)
     {
       GdkColorState *cs;
 
-      cs = gdk_color_state_new_for_cicp (&(GdkCicp) { 1, 13, 6, 0 }, NULL);
+      cs = gdk_color_state_new_for_cicp (&(GdkCicp) { 1, 13, 5, 0 }, NULL);
       g_assert (cs);
 
       g_once_init_leave_pointer (&yuv_cs, cs);
