@@ -18,8 +18,7 @@
 /**
  * GdkContentFormats:
  *
- * The `GdkContentFormats` structure is used to advertise and negotiate the
- * format of content.
+ * Used to advertise and negotiate the format of content.
  *
  * You will encounter `GdkContentFormats` when interacting with objects
  * controlling operations that pass data between different widgets, window
@@ -52,6 +51,12 @@
  * endeavor.
  */
 
+/**
+ * GdkContentFormatsBuilder:
+ *
+ * Creates `GdkContentFormats` objects.
+ */
+
 #include "config.h"
 
 #include "gdkcontentformats.h"
@@ -572,6 +577,24 @@ gdk_content_formats_get_mime_types (const GdkContentFormats *formats,
 }
 
 /**
+ * gdk_content_formats_is_empty:
+ * @formats: content formats
+ *
+ * Returns whether the content formats contain any formats.
+ *
+ * Returns: true if @formats contains no mime types and no GTypes
+ *
+ * Since: 4.18
+ */
+gboolean
+gdk_content_formats_is_empty (GdkContentFormats *formats)
+{
+  return formats->n_mime_types == 0 && formats->n_gtypes == 0;
+}
+
+/* {{{ GdkContentFormatsBuilder */
+
+/*
  * GdkContentFormatsBuilder:
  *
  * A `GdkContentFormatsBuilder` is an auxiliary struct used to create
@@ -645,6 +668,9 @@ gdk_content_formats_builder_clear (GdkContentFormatsBuilder *builder)
 {
   g_clear_pointer (&builder->gtypes, g_slist_free);
   g_clear_pointer (&builder->mime_types, g_slist_free);
+
+  builder->n_gtypes = 0;
+  builder->n_mime_types = 0;
 }
 
 /**
@@ -822,6 +848,7 @@ gdk_content_formats_builder_add_mime_type (GdkContentFormatsBuilder *builder,
   builder->n_mime_types++;
 }
 
+/* }}} */
 /* {{{ GdkFileList */
 
 /* We're using GdkFileList* and GSList* interchangeably, counting on the
@@ -909,3 +936,5 @@ gdk_file_list_new_from_array (GFile **files,
 }
 
 /* }}} */
+
+/* vim:set foldmethod=marker: */
