@@ -21,7 +21,7 @@
 
 #include "gdkcontentserializer.h"
 
-#include "gdkcontentformats.h"
+#include "gdkcontentformatsprivate.h"
 #include "deprecated/gdkpixbuf.h"
 #include "filetransferportalprivate.h"
 #include "gdktextureprivate.h"
@@ -39,8 +39,7 @@
 /**
  * GdkContentSerializer:
  *
- * A `GdkContentSerializer` is used to serialize content for
- * inter-application data transfers.
+ * Serializes content for inter-application data transfers.
  *
  * The `GdkContentSerializer` transforms an object that is identified
  * by a GType into a serialized form (i.e. a byte stream) that is
@@ -66,8 +65,6 @@ struct _Serializer
 
 GQueue serializers = G_QUEUE_INIT;
 
-static void init (void);
-
 #define GDK_CONTENT_SERIALIZER_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GDK_TYPE_CONTENT_SERIALIZER, GdkContentSerializerClass))
 #define GDK_IS_CONTENT_SERIALIZER_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GDK_TYPE_CONTENT_SERIALIZER))
 #define GDK_CONTENT_SERIALIZER_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GDK_TYPE_CONTENT_SERIALIZER, GdkContentSerializerClass))
@@ -403,6 +400,10 @@ gdk_content_serializer_return_error (GdkContentSerializer *serializer,
  * @notify: destroy notify for @data
  *
  * Registers a function to serialize objects of a given type.
+ *
+ * Since 4.20, when looking up a serializer to use, GTK will
+ * use the last registered serializer for a given mime type,
+ * so applications can override the built-in serializers.
  */
 void
 gdk_content_register_serializer (GType                    type,
@@ -435,11 +436,9 @@ lookup_serializer (const char *mime_type,
 
   g_return_val_if_fail (mime_type != NULL, NULL);
 
-  init ();
-
   mime_type = g_intern_string (mime_type);
 
-  for (l = g_queue_peek_head_link (&serializers); l; l = l->next)
+  for (l = g_queue_peek_tail_link (&serializers); l; l = l->prev)
     {
       Serializer *serializer = l->data;
 
@@ -468,8 +467,6 @@ gdk_content_formats_union_serialize_gtypes (GdkContentFormats *formats)
 
   g_return_val_if_fail (formats != NULL, NULL);
 
-  init ();
-
   builder = gdk_content_formats_builder_new ();
   gdk_content_formats_builder_add_formats (builder, formats);
 
@@ -503,8 +500,6 @@ gdk_content_formats_union_serialize_mime_types (GdkContentFormats *formats)
 
   g_return_val_if_fail (formats != NULL, NULL);
 
-  init ();
-
   builder = gdk_content_formats_builder_new ();
   gdk_content_formats_builder_add_formats (builder, formats);
 
@@ -539,8 +534,8 @@ serialize_not_found (GdkContentSerializer *serializer)
  * @value: the content to serialize
  * @io_priority: the I/O priority of the operation
  * @cancellable: (nullable): optional `GCancellable` object
- * @callback: (scope async): callback to call when the operation is done
- * @user_data: (closure): data to pass to the callback function
+ * @callback: (scope async) (closure): callback to call when the operation is done
+ * @user_data: data to pass to the callback function
  *
  * Serialize content and write it to the given output stream, asynchronously.
  *
@@ -692,7 +687,7 @@ serialize_texture_in_thread (GTask        *task,
   texture = g_value_get_object (value);
 
   if (strcmp (gdk_content_serializer_get_mime_type (serializer), "image/png") == 0)
-    bytes = gdk_save_png (texture);
+    bytes = gdk_save_png (texture, NULL);
   else if (strcmp (gdk_content_serializer_get_mime_type (serializer), "image/tiff") == 0)
     bytes = gdk_save_tiff (texture);
   else if (strcmp (gdk_content_serializer_get_mime_type (serializer), "image/jpeg") == 0)
@@ -794,11 +789,25 @@ file_serializer_finish (GObject      *source,
     gdk_content_serializer_return_success (serializer);
 }
 
+
+static const char *
+file_force_uri_schemas[] = { "content", "http", "https" };
 static char *
 file_get_native_uri (GFile *file)
 {
   char *path;
 
+  /*
+   * We want to convert URIs to file:// URIs, so other applications
+   * that do not use GIO/GVFS can read text/uri-list from the clipboard.
+   *
+   * But we don't want to do that for URIs that are widely understood,
+   * so for lack of better APIs, we whitelist those.
+   */
+  for (gsize i = 0; i < G_N_ELEMENTS (file_force_uri_schemas); i++)
+    if (g_file_has_uri_scheme (file, file_force_uri_schemas[i]))
+        return g_file_get_uri (file);
+
   path = g_file_get_path (file);
   if (path != NULL)
     {
@@ -953,8 +962,8 @@ color_serializer (GdkContentSerializer *serializer)
   gdk_content_serializer_set_task_data (serializer, data, g_free);
 }
 
-static void
-init (void)
+void
+gdk_content_init_serializers (void)
 {
   static gboolean initialized = FALSE;
   GSList *formats, *f;
@@ -965,6 +974,8 @@ init (void)
 
   initialized = TRUE;
 
+  /* Textures */
+
   gdk_content_register_serializer (GDK_TYPE_TEXTURE,
                                    "image/png",
                                    texture_serializer,
@@ -1016,37 +1027,41 @@ init (void)
 
   g_slist_free (formats);
 
+  /* Files */
+
+  gdk_content_register_serializer (G_TYPE_FILE,
+                                   "text/plain;charset=utf-8",
+                                   file_text_serializer,
+                                   NULL,
+                                   NULL);
+  gdk_content_register_serializer (G_TYPE_FILE,
+                                   "text/uri-list",
+                                   file_uri_serializer,
+                                   NULL,
+                                   NULL);
+  gdk_content_register_serializer (GDK_TYPE_FILE_LIST,
+                                   "text/plain;charset=utf-8",
+                                   file_text_serializer,
+                                   NULL,
+                                   NULL);
+  gdk_content_register_serializer (GDK_TYPE_FILE_LIST,
+                                   "text/uri-list",
+                                   file_uri_serializer,
+                                   NULL,
+                                   NULL);
+
 #if defined(G_OS_UNIX) && !defined(__APPLE__)
   file_transfer_portal_register ();
 #endif
 
-  gdk_content_register_serializer (G_TYPE_FILE,
-                                   "text/uri-list",
-                                   file_uri_serializer,
-                                   NULL,
-                                   NULL);
-  gdk_content_register_serializer (G_TYPE_FILE,
-                                   "text/plain;charset=utf-8",
-                                   file_text_serializer,
-                                   NULL,
-                                   NULL);
-
-  gdk_content_register_serializer (GDK_TYPE_FILE_LIST,
-                                   "text/uri-list",
-                                   file_uri_serializer,
-                                   NULL,
-                                   NULL);
-  gdk_content_register_serializer (GDK_TYPE_FILE_LIST,
-                                   "text/plain;charset=utf-8",
-                                   file_text_serializer,
-                                   NULL,
-                                   NULL);
+  /* Strings */
 
   gdk_content_register_serializer (G_TYPE_STRING,
                                    "text/plain;charset=utf-8",
                                    string_serializer,
                                    (gpointer) "utf-8",
                                    NULL);
+
   if (!g_get_charset (&charset))
     {
       char *mime = g_strdup_printf ("text/plain;charset=%s", charset);
@@ -1057,12 +1072,15 @@ init (void)
                                        NULL);
       g_free (mime);
     }
+
   gdk_content_register_serializer (G_TYPE_STRING,
                                    "text/plain",
                                    string_serializer,
                                    (gpointer) "ASCII",
                                    NULL);
 
+  /* Colors */
+
   gdk_content_register_serializer (GDK_TYPE_RGBA,
                                    "application/x-color",
                                    color_serializer,
