@@ -22,8 +22,7 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GDK_CURSOR_PRIVATE_H__
-#define __GDK_CURSOR_PRIVATE_H__
+#pragma once
 
 #include <gdk/gdkcursor.h>
 
@@ -39,12 +38,15 @@ struct _GdkCursor
 {
   GObject parent_instance;
 
-  GdkDisplay *display;
   GdkCursor *fallback;
   char *name;
   GdkTexture *texture;
   int hotspot_x;
   int hotspot_y;
+
+  GdkCursorGetTextureCallback callback;
+  gpointer data;
+  GDestroyNotify destroy;
 };
 
 struct _GdkCursorClass
@@ -56,6 +58,13 @@ guint                   gdk_cursor_hash                         (gconstpointer
 gboolean                gdk_cursor_equal                        (gconstpointer          a,
                                                                  gconstpointer          b);
 
+GdkTexture *            gdk_cursor_get_texture_for_size         (GdkCursor *cursor,
+                                                                 int        cursor_size,
+                                                                 double     scale,
+                                                                 int       *width,
+                                                                 int       *height,
+                                                                 int       *hotspot_x,
+                                                                 int       *hotspot_y);
+
 G_END_DECLS
 
-#endif /* __GDK_CURSOR_PRIVATE_H__ */
