@@ -29,8 +29,7 @@
 /**
  * GdkFrameClock:
  *
- * A `GdkFrameClock` tells the application when to update and repaint
- * a surface.
+ * Tells the application when to update and repaint a surface.
  *
  * This may be synced to the vertical refresh rate of the monitor, for example.
  * Even when the frame clock uses a simple timer rather than a hardware-based
@@ -395,6 +394,12 @@ _gdk_frame_clock_uninhibit_freeze (GdkFrameClock *clock)
     _gdk_frame_clock_freeze (clock);
 }
 
+gboolean
+gdk_frame_clock_is_frozen (GdkFrameClock *clock)
+{
+  return clock->priv->n_freeze_inhibitors == 0;
+}
+
 static inline gint64
 _gdk_frame_clock_get_frame_counter (GdkFrameClock *frame_clock)
 {
