@@ -19,7 +19,6 @@
 
 #pragma once
 
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include "gdkenumtypes.h"
 #include "gdkmemoryformatprivate.h"
 #include "gdksurface.h"
@@ -93,7 +92,6 @@ struct _GdkSurface
 
   GdkFrameClock *frame_clock; /* NULL to use from parent or default */
 
-  GSList *draw_contexts;
   GdkDrawContext *paint_context;
 
   GdkSeat *current_shortcuts_inhibited_seat;
@@ -304,12 +302,6 @@ void gdk_surface_get_geometry (GdkSurface *surface,
 
 void                    gdk_surface_set_frame_clock             (GdkSurface             *surface,
                                                                  GdkFrameClock          *clock);
-void                    gdk_surface_set_egl_native_window       (GdkSurface             *self,
-                                                                 gpointer                native_window);
-GdkMemoryDepth          gdk_surface_ensure_egl_surface          (GdkSurface             *self,
-                                                                 GdkMemoryDepth          depth);
-gpointer /*EGLSurface*/ gdk_surface_get_egl_surface             (GdkSurface             *self);
-
 gboolean                gdk_surface_get_gl_is_srgb              (GdkSurface             *self);
 
 void                    gdk_surface_set_widget                  (GdkSurface             *self,
@@ -359,8 +351,11 @@ gsize           gdk_surface_get_n_subsurfaces  (GdkSurface          *surface);
 GdkSubsurface * gdk_surface_get_subsurface     (GdkSurface          *surface,
                                                 gsize                idx);
 
-GdkColorState * gdk_surface_get_color_state    (GdkSurface          *surface);
-void            gdk_surface_set_color_state    (GdkSurface          *surface,
-                                                GdkColorState       *color_state);
+GdkColorState *         gdk_surface_get_color_state                     (GdkSurface             *surface);
+void                    gdk_surface_set_color_state                     (GdkSurface             *surface,
+                                                                         GdkColorState          *color_state);
+void                    gdk_surface_set_attached_context                (GdkSurface             *self,
+                                                                         GdkDrawContext         *context);
+GdkDrawContext *        gdk_surface_get_attached_context                (GdkSurface             *self);
 
 G_END_DECLS
