@@ -245,7 +245,7 @@ gdk_png_get_color_state (png_struct  *png,
   return GDK_COLOR_STATE_SRGB;
 }
 
-static void
+static GdkColorState *
 gdk_png_set_color_state (png_struct    *png,
                          png_info      *info,
                          GdkColorState *color_state,
@@ -289,6 +289,8 @@ gdk_png_set_color_state (png_struct    *png,
   /* For good measure, we add an sRGB chunk too */
   if (gdk_color_state_equal (color_state, GDK_COLOR_STATE_SRGB))
     png_set_sRGB (png, info, PNG_sRGB_INTENT_PERCEPTUAL);
+
+  return color_state;
 }
 
 /* }}} */
@@ -502,7 +504,8 @@ gdk_load_png (GBytes      *bytes,
 }
 
 GBytes *
-gdk_save_png (GdkTexture *texture)
+gdk_save_png (GdkTexture *texture,
+              GHashTable *options)
 {
   png_struct *png = NULL;
   png_info *info;
@@ -518,6 +521,7 @@ gdk_save_png (GdkTexture *texture)
   int png_format;
   int depth;
   png_byte chunk_data[4];
+  png_textp text_ptr = NULL;
 
   width = gdk_texture_get_width (texture);
   height = gdk_texture_get_height (texture);
@@ -587,6 +591,15 @@ gdk_save_png (GdkTexture *texture)
     case GDK_MEMORY_G10X6_B10X6R10X6_420:
     case GDK_MEMORY_G12X4_B12X4R12X4_420:
     case GDK_MEMORY_G16_B16R16_420:
+    case GDK_MEMORY_X6G10_X6B10_X6R10_420:
+    case GDK_MEMORY_X6G10_X6B10_X6R10_422:
+    case GDK_MEMORY_X6G10_X6B10_X6R10_444:
+    case GDK_MEMORY_X4G12_X4B12_X4R12_420:
+    case GDK_MEMORY_X4G12_X4B12_X4R12_422:
+    case GDK_MEMORY_X4G12_X4B12_X4R12_444:
+    case GDK_MEMORY_G16_B16_R16_420:
+    case GDK_MEMORY_G16_B16_R16_422:
+    case GDK_MEMORY_G16_B16_R16_444:
       format = GDK_MEMORY_R16G16B16;
       png_format = PNG_COLOR_TYPE_RGB;
       depth = 16;
@@ -668,16 +681,52 @@ gdk_save_png (GdkTexture *texture)
                 PNG_COMPRESSION_TYPE_DEFAULT,
                 PNG_FILTER_TYPE_DEFAULT);
 
-  gdk_png_set_color_state (png, info, color_state, chunk_data);
+  color_state = gdk_png_set_color_state (png, info, color_state, chunk_data);
 
   png_write_info (png, info);
 
+  if (options)
+    {
+      GHashTableIter iter;
+      char *key, *value;
+      GArray *text_data;
+      int n_keys;
+
+      text_data = g_array_sized_new (FALSE, TRUE, sizeof (png_text), g_hash_table_size (options));
+      g_hash_table_iter_init (&iter, options);
+
+      while (g_hash_table_iter_next (&iter, (gpointer *) &key, (gpointer *) &value))
+        {
+          png_text text;
+          int len;
+
+          len = strlen (key);
+          if (len < 1 || len > 79)
+            continue;
+
+          text.key = key;
+          text.compression = PNG_TEXT_COMPRESSION_NONE;
+          text.text = value;
+          text.text_length = strlen (value);
+          text.itxt_length = 0;
+          text.lang = NULL;
+          text.lang_key = NULL;
+
+          g_array_append_val (text_data, text);
+        }
+
+      n_keys = text_data->len;
+      text_ptr = (png_textp) g_array_free (text_data, FALSE);
+      png_set_text (png, info, text_ptr, n_keys);
+    }
+
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
   png_set_swap (png);
 #endif
 
   gdk_texture_downloader_init (&downloader, texture);
   gdk_texture_downloader_set_format (&downloader, format);
+  gdk_texture_downloader_set_color_state (&downloader, color_state);
   bytes = gdk_texture_downloader_download_bytes (&downloader, &stride);
   gdk_texture_downloader_finish (&downloader);
   data = g_bytes_get_data (bytes, NULL);
@@ -692,6 +741,8 @@ gdk_save_png (GdkTexture *texture)
   gdk_color_state_unref (color_state);
   g_bytes_unref (bytes);
 
+  g_free (text_ptr);
+
   return g_bytes_new_take (io.data, io.size);
 }
 
