@@ -42,12 +42,28 @@
 #include "gdkmonitorprivate.h"
 #include "gdksurfaceprivate.h"
 
-#ifndef AVAILABLE_MAC_OS_X_VERSION_10_15_AND_LATER
-typedef NSString *CALayerContentsGravity;
-#endif
-
 @implementation GdkMacosWindow
 
+static Class _contentViewClass = nil;
+
++(void)setContentViewClass:(Class)newViewClass
+{
+    GDK_DEBUG (MISC, "Setting new content view class to %s", [[newViewClass description] UTF8String]);
+
+    if (newViewClass == nil  || [newViewClass isSubclassOfClass:[GdkMacosView class]])
+      _contentViewClass = newViewClass;
+    else
+      g_critical ("Assigned content view class %s is not a subclass of GdkMacosView", [[newViewClass description] UTF8String]);
+}
+
++(Class)contentViewClass
+{
+    if (_contentViewClass != nil)
+      return _contentViewClass;
+
+    return [GdkMacosView class];
+}
+
 -(BOOL)windowShouldClose:(id)sender
 {
   GdkDisplay *display;
@@ -139,15 +155,13 @@ typedef NSString *CALayerContentsGravity;
   switch ((int)event_type)
     {
     case NSEventTypeLeftMouseUp: {
-      GdkDisplay *display = gdk_surface_get_display (GDK_SURFACE (gdk_surface));
-      double time = ((double)[event timestamp]) * 1000.0;
+      if (inManualMove || inManualResize || inMove)
+        _gdk_macos_display_send_event ([self gdkDisplay], event);
 
       inManualMove = NO;
       inManualResize = NO;
       inMove = NO;
 
-      _gdk_macos_display_break_all_grabs (GDK_MACOS_DISPLAY (display), time);
-
       /* Reset gravity */
       [[[self contentView] layer] setContentsGravity:kCAGravityBottomLeft];
 
@@ -155,7 +169,7 @@ typedef NSString *CALayerContentsGravity;
     }
 
     case NSEventTypeLeftMouseDragged:
-      if ([self trackManualMove] || [self trackManualResize])
+      if (inManualMove || [self trackManualResize])
         return;
       break;
 
@@ -225,7 +239,7 @@ typedef NSString *CALayerContentsGravity;
   [self setReleasedWhenClosed:YES];
   [self setPreservesContentDuringLiveResize:NO];
 
-  view = [[GdkMacosView alloc] initWithFrame:contentRect];
+  view = [[[GdkMacosWindow contentViewClass] alloc] initWithFrame:contentRect];
   [self setContentView:view];
   [view release];
 
@@ -280,63 +294,6 @@ typedef NSString *CALayerContentsGravity;
     [self windowDidResignKey:nil];
 }
 
--(BOOL)trackManualMove
-{
-  NSRect windowFrame;
-  NSPoint currentLocation;
-  GdkMonitor *monitor;
-  GdkRectangle geometry;
-  GdkRectangle workarea;
-  GdkRectangle window_gdk;
-  GdkPoint pointer_position;
-  GdkPoint new_origin;
-
-  if (!inManualMove)
-    return NO;
-
-  windowFrame = [self frame];
-  currentLocation = [NSEvent mouseLocation];
-
-  /* Update the snapping geometry to match the current monitor */
-  monitor = _gdk_macos_display_get_monitor_at_display_coords ([self gdkDisplay],
-                                                              currentLocation.x,
-                                                              currentLocation.y);
-  gdk_monitor_get_geometry (monitor, &geometry);
-  gdk_macos_monitor_get_workarea (monitor, &workarea);
-  _edge_snapping_set_monitor (&self->snapping, &geometry, &workarea);
-
-  /* Convert origins to GDK coordinates */
-  _gdk_macos_display_from_display_coords ([self gdkDisplay],
-                                          currentLocation.x,
-                                          currentLocation.y,
-                                          &pointer_position.x,
-                                          &pointer_position.y);
-  _gdk_macos_display_from_display_coords ([self gdkDisplay],
-                                          windowFrame.origin.x,
-                                          windowFrame.origin.y + windowFrame.size.height,
-                                          &window_gdk.x,
-                                          &window_gdk.y);
-  window_gdk.width = windowFrame.size.width;
-  window_gdk.height = windowFrame.size.height;
-
-  /* Now place things on the monitor */
-  _edge_snapping_motion (&self->snapping, &pointer_position, &window_gdk);
-
-  /* Convert to quartz coordinates */
-  _gdk_macos_display_to_display_coords ([self gdkDisplay],
-                                        window_gdk.x,
-                                        window_gdk.y + window_gdk.height,
-                                        &new_origin.x, &new_origin.y);
-  windowFrame.origin.x = new_origin.x;
-  windowFrame.origin.y = new_origin.y;
-
-  [self setFrame:NSMakeRect (new_origin.x, new_origin.y,
-                             window_gdk.width, window_gdk.height)
-         display:YES];
-
-  return YES;
-}
-
 -(void)windowDidMove:(NSNotification *)notification
 {
   if ([self isZoomed])
@@ -370,40 +327,22 @@ typedef NSString *CALayerContentsGravity;
 
 -(void)beginManualMove
 {
-  NSPoint initialMoveLocation;
-  GdkPoint point;
-  GdkMonitor *monitor;
-  GdkRectangle geometry;
-  GdkRectangle area;
-  GdkRectangle workarea;
+  NSEvent *event;
 
   if (inMove || inManualMove || inManualResize)
     return;
 
+  event = _gdk_macos_display_get_last_nsevent ();
+
+  if (event == NULL)
+    return; 
+  
   inManualMove = YES;
 
-  monitor = _gdk_macos_surface_get_best_monitor ([self gdkSurface]);
-  gdk_monitor_get_geometry (monitor, &geometry);
-  gdk_macos_monitor_get_workarea (monitor, &workarea);
-
-  initialMoveLocation = [NSEvent mouseLocation];
-
-  _gdk_macos_display_from_display_coords ([self gdkDisplay],
-                                          initialMoveLocation.x,
-                                          initialMoveLocation.y,
-                                          &point.x,
-                                          &point.y);
-
-  area.x = gdk_surface->root_x;
-  area.y = gdk_surface->root_y;
-  area.width = GDK_SURFACE (gdk_surface)->width;
-  area.height = GDK_SURFACE (gdk_surface)->height;
-
-  _edge_snapping_init (&self->snapping,
-                       &geometry,
-                       &workarea,
-                       &point,
-                       &area);
+  /* The docs state it has to be a button press event,
+   * but it works just as well with motion events.
+   */
+  [self performWindowDragWithEvent: event];
 }
 
 -(BOOL)trackManualResize
@@ -720,6 +659,9 @@ typedef NSString *CALayerContentsGravity;
 {
   inFullscreenTransition = NO;
   initialPositionKnown = NO;
+
+  [self updateToolbarAppearence];
+
   [self checkSendEnterNotify];
 }
 
@@ -732,6 +674,9 @@ typedef NSString *CALayerContentsGravity;
 {
   inFullscreenTransition = NO;
   initialPositionKnown = NO;
+
+  [self updateToolbarAppearence];
+
   [self checkSendEnterNotify];
 }
 
@@ -750,6 +695,13 @@ typedef NSString *CALayerContentsGravity;
   _gdk_macos_surface_monitor_changed (gdk_surface);
 }
 
+/* We need this for the Vulkan renderer to update it's content scaling.
+ */
+-(void)windowDidChangeBackingProperties:(NSNotification *) notification
+{
+  _gdk_surface_update_size (GDK_SURFACE (gdk_surface));
+}
+
 -(void)setGdkSurface:(GdkMacosSurface *)surface
 {
   self->gdk_surface = surface;
@@ -762,20 +714,77 @@ typedef NSString *CALayerContentsGravity;
   if (decorated)
     {
       style_mask &= ~NSWindowStyleMaskFullSizeContentView;
-      [self setTitleVisibility:NSWindowTitleVisible];
     }
   else
     {
       style_mask |= NSWindowStyleMaskFullSizeContentView;
-      [self setTitleVisibility:NSWindowTitleHidden];
     }
 
-  [self setTitlebarAppearsTransparent:!decorated];
-  [[self standardWindowButton:NSWindowCloseButton] setHidden:!decorated];
-  [[self standardWindowButton:NSWindowMiniaturizeButton] setHidden:!decorated];
-  [[self standardWindowButton:NSWindowZoomButton] setHidden:!decorated];
-
   [self setStyleMask:style_mask];
+
+  [self updateToolbarAppearence];
+}
+
+-(BOOL)showStandardWindowButtons;
+{
+  return showButtonCount > 0;
+}
+
+-(void)setShowStandardWindowButtons:(BOOL)show
+{
+  if (show)
+    showButtonCount++;
+  else if (showButtonCount > 0)
+    showButtonCount--;
+  else
+    {
+      g_warning ("Show standard window button count doesn't match hide count");
+      return;
+    }
+
+  [self updateToolbarAppearence];
+}
+
+/* updateToolbarAppearence:
+ * Update the toolbar appearence based on the following criteria:
+ *
+ * 1. The window is used Client Side Decorations (style mask is set)
+ * 2. The window has native window buttons enabled
+ * 3. The window is in fullscreen mode
+ */
+-(void)updateToolbarAppearence
+{
+  NSWindowStyleMask style_mask = [self styleMask];
+  BOOL is_fullscreen = (style_mask & NSWindowStyleMaskFullScreen) != 0;
+  BOOL is_csd = !is_fullscreen && (style_mask & NSWindowStyleMaskFullSizeContentView) != 0;
+  BOOL hidden = is_csd && (showButtonCount == 0);
+
+  /* Do not update toolbars when in fullscreen transition, as it can cause the app to crash.
+   * It will be updated once the transition has finished.
+   */
+  if (inFullscreenTransition)
+    return;
+
+  /* By assigning a toolbar, the window controls are moved a bit more inwards,
+   * In line with how toolbars look in macOS apps.
+   * I haven't found a better way. Unfortunately we have to be careful not to
+   * update the toolbar during a fullscreen transition.
+   */
+  if (is_csd && (showButtonCount > 0) && [self toolbar] == nil)
+    {
+      NSToolbar *toolbar = [[NSToolbar alloc] init];
+      [self setToolbar:toolbar];
+      [toolbar release];
+    }
+  else if (!is_csd && [self toolbar] != nil)
+    [self setToolbar:nil];
+
+  [self setTitleVisibility:is_csd ? NSWindowTitleHidden : NSWindowTitleVisible];
+  [self setTitlebarAppearsTransparent:is_csd];
+
+  [[self standardWindowButton:NSWindowCloseButton] setHidden:hidden];
+  [[self standardWindowButton:NSWindowMiniaturizeButton] setHidden:hidden];
+  [[self standardWindowButton:NSWindowZoomButton] setHidden:hidden];
 }
 
 -(GdkMacosSurface *)gdkSurface
