@@ -26,7 +26,7 @@
 #include <stdint.h>
 #include <wayland-client.h>
 #include <wayland-egl.h>
-#include <gdk/wayland/tablet-unstable-v2-client-protocol.h>
+#include <gdk/wayland/tablet-v2-client-protocol.h>
 #include <gdk/wayland/gtk-shell-client-protocol.h>
 #include <gdk/wayland/xdg-shell-client-protocol.h>
 #include <gdk/wayland/xdg-shell-unstable-v6-client-protocol.h>
@@ -89,7 +89,7 @@ struct _GdkWaylandDisplay
   gboolean list_protocols;
   char **skip_protocols;
 
-  guint32    shell_capabilities;
+  uint32_t shell_capabilities;
 
   /* Startup notification */
   char *startup_notification_id;
