@@ -30,6 +30,8 @@
 
 #include <epoxy/glx.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 struct _GdkX11GLContextGLX
 {
   GdkX11GLContext parent_instance;
@@ -84,10 +86,13 @@ gdk_x11_surface_get_glx_drawable (GdkSurface *surface)
 void
 gdk_x11_surface_destroy_glx_drawable (GdkX11Surface *self)
 {
+  GdkGLContext *context;
+
   if (self->glx_drawable == None)
     return;
 
-  gdk_gl_context_clear_current_if_surface (GDK_SURFACE (self));
+  context = gdk_gl_context_clear_current_if_surface (GDK_SURFACE (self));
+  g_clear_object (&context);
 
   glXDestroyWindow (gdk_x11_display_get_xdisplay (gdk_surface_get_display (GDK_SURFACE (self))),
                     self->glx_drawable);
@@ -122,6 +127,7 @@ maybe_wait_for_vblank (GdkDisplay  *display,
 
 static void
 gdk_x11_gl_context_glx_end_frame (GdkDrawContext *draw_context,
+                                  gpointer        context_data,
                                   cairo_region_t *painted)
 {
   GdkX11GLContextGLX *self = GDK_X11_GL_CONTEXT_GLX (draw_context);
@@ -133,7 +139,7 @@ gdk_x11_gl_context_glx_end_frame (GdkDrawContext *draw_context,
   GdkX11Display *display_x11 = GDK_X11_DISPLAY (display);
   GLXDrawable drawable;
 
-  GDK_DRAW_CONTEXT_CLASS (gdk_x11_gl_context_glx_parent_class)->end_frame (draw_context, painted);
+  GDK_DRAW_CONTEXT_CLASS (gdk_x11_gl_context_glx_parent_class)->end_frame (draw_context, context_data, painted);
 
   gdk_gl_context_make_current (context);
 
@@ -916,6 +922,8 @@ gdk_x11_display_create_glx_config (GdkX11Display  *self,
  * Retrieves the version of the GLX implementation.
  *
  * Returns: %TRUE if GLX is available
+ *
+ * Deprecated: 4.18
  */
 gboolean
 gdk_x11_display_get_glx_version (GdkDisplay *display,
