@@ -46,6 +46,8 @@
 #include <X11/extensions/Xfixes.h>
 #endif
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static void gdk_x11_screen_dispose  (GObject      *object);
 static void gdk_x11_screen_finalize (GObject      *object);
 static void init_randr_support	    (GdkX11Screen *screen);
@@ -119,6 +121,8 @@ gdk_x11_screen_finalize (GObject *object)
  * extension, 0 is returned.
  *
  * Returns: the XID of the monitor
+ *
+ * Deprecated: 4.18
  */
 XID
 gdk_x11_screen_get_monitor_output (GdkX11Screen *x11_screen,
@@ -354,6 +358,8 @@ out:
  * Returns the screen of a `GdkX11Screen`.
  *
  * Returns: (transfer none): an Xlib Screen*
+ *
+ * Deprecated: 4.18
  */
 Screen *
 gdk_x11_screen_get_xscreen (GdkX11Screen *screen)
@@ -369,6 +375,8 @@ gdk_x11_screen_get_xscreen (GdkX11Screen *screen)
  *
  * Returns: the position of @screen among the screens
  *   of its display
+ *
+ * Deprecated: 4.18
  */
 int
 gdk_x11_screen_get_screen_number (GdkX11Screen *screen)
@@ -492,9 +500,18 @@ init_randr15 (GdkX11Screen *x11_screen)
 
       if (output_info->crtc)
         {
-          XRRCrtcInfo *crtc = XRRGetCrtcInfo (x11_screen->xdisplay, resources, output_info->crtc);
+          XRRCrtcInfo *crtc;
           int j;
 
+          gdk_x11_display_error_trap_push (display);
+          crtc = XRRGetCrtcInfo (x11_screen->xdisplay, resources,
+                                 output_info->crtc);
+          if (gdk_x11_display_error_trap_pop (display))
+            {
+              XRRFreeOutputInfo (output_info);
+              continue;
+            }
+
           for (j = 0; j < resources->nmode; j++)
             {
               XRRModeInfo *xmode = &resources->modes[j];
@@ -509,19 +526,6 @@ init_randr15 (GdkX11Screen *x11_screen)
           XRRFreeCrtcInfo (crtc);
         }
 
-      monitor = find_monitor_by_output (x11_display, output);
-      if (monitor)
-        monitor->remove = FALSE;
-      else
-        {
-          monitor = g_object_new (GDK_TYPE_X11_MONITOR,
-                                  "display", display,
-                                  NULL);
-          monitor->output = output;
-          monitor->add = TRUE;
-          g_list_store_append (x11_display->monitors, monitor);
-        }
-
       /* Fetch minimal manufacturer information (PNP ID) from EDID */
       {
         #define EDID_LENGTH 128
@@ -534,6 +538,7 @@ init_randr15 (GdkX11Screen *x11_screen)
 
         edid_atom = XInternAtom (disp, RR_PROPERTY_RANDR_EDID, FALSE);
 
+        gdk_x11_display_error_trap_push (display);
         XRRGetOutputProperty (disp, output,
                               edid_atom,
                               0,
@@ -546,6 +551,11 @@ init_randr15 (GdkX11Screen *x11_screen)
                               &nbytes,
                               &bytes_left,
                               &prop);
+        if (gdk_x11_display_error_trap_pop (display))
+          {
+            XRRFreeOutputInfo (output_info);
+            continue;
+          }
 
         // Check partial EDID header (whole header: 00 ff ff ff ff ff ff 00)
         if (nbytes >= EDID_LENGTH && prop[0] == 0x00 && prop[1] == 0xff)
@@ -566,6 +576,19 @@ init_randr15 (GdkX11Screen *x11_screen)
         #undef EDID_LENGTH
       }
 
+      monitor = find_monitor_by_output (x11_display, output);
+      if (monitor)
+        monitor->remove = FALSE;
+      else
+        {
+          monitor = g_object_new (GDK_TYPE_X11_MONITOR,
+                                  "display", display,
+                                  NULL);
+          monitor->output = output;
+          monitor->add = TRUE;
+          g_list_store_append (x11_display->monitors, monitor);
+        }
+
       gdk_monitor_get_geometry (GDK_MONITOR (monitor), &geometry);
       name = g_strndup (output_info->name, output_info->nameLen);
 
@@ -670,8 +693,13 @@ init_randr13 (GdkX11Screen *x11_screen)
   for (i = 0; i < resources->noutput; ++i)
     {
       RROutput output = resources->outputs[i];
-      XRROutputInfo *output_info =
-        XRRGetOutputInfo (x11_screen->xdisplay, resources, output);
+      XRROutputInfo *output_info;
+
+      gdk_x11_display_error_trap_push (display);
+      output_info = XRRGetOutputInfo (x11_screen->xdisplay, resources, output);
+
+      if (gdk_x11_display_error_trap_pop (display))
+        continue;
 
       if (output_info->connection == RR_Disconnected)
         {
@@ -682,13 +710,22 @@ init_randr13 (GdkX11Screen *x11_screen)
       if (output_info->crtc)
 	{
 	  GdkX11Monitor *monitor;
-	  XRRCrtcInfo *crtc = XRRGetCrtcInfo (x11_screen->xdisplay, resources, output_info->crtc);
+          XRRCrtcInfo *crtc;
           char *name;
           GdkRectangle geometry;
           GdkRectangle newgeo;
           int j;
           int refresh_rate = 0;
 
+          gdk_x11_display_error_trap_push (display);
+          crtc = XRRGetCrtcInfo (x11_screen->xdisplay, resources, output_info->crtc);
+
+          if (gdk_x11_display_error_trap_pop (display))
+            {
+              XRRFreeOutputInfo (output_info);
+              continue;
+            }
+
           for (j = 0; j < resources->nmode; j++)
             {
               XRRModeInfo *xmode = &resources->modes[j];
@@ -760,8 +797,10 @@ init_randr13 (GdkX11Screen *x11_screen)
     }
 
   x11_display->primary_monitor = 0;
+  gdk_x11_display_error_trap_push (display);
   primary_output = XRRGetOutputPrimary (x11_screen->xdisplay,
                                         x11_screen->xroot_window);
+  gdk_x11_display_error_trap_pop_ignored (display);
 
   for (i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (x11_display->monitors)); i++)
     {
@@ -1114,7 +1153,6 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
 {
   GdkDisplay *display;
   Window window;
-  guint64 now;
   int error;
 
   display = x11_screen->display;
@@ -1124,11 +1162,6 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
   if (x11_screen->wmspec_check_window != None)
     return; /* already have it */
 
-  now = g_get_monotonic_time ();
-
-  if ((now - x11_screen->last_wmspec_check_time) / 1e6 < 15)
-    return; /* we've checked recently */
-
   window = get_net_supporting_wm_check (x11_screen, x11_screen->xroot_window);
   if (window == None)
     return;
@@ -1153,7 +1186,6 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
         return;
 
       x11_screen->wmspec_check_window = window;
-      x11_screen->last_wmspec_check_time = now;
       x11_screen->need_refetch_net_supported = TRUE;
       x11_screen->need_refetch_wm_name = TRUE;
 
@@ -1181,6 +1213,8 @@ fetch_net_wm_check_window (GdkX11Screen *x11_screen)
  * a window manager change.
  *
  * Returns: %TRUE if the window manager supports @property
+ *
+ * Deprecated: 4.18
  **/
 gboolean
 gdk_x11_screen_supports_net_wm_hint (GdkX11Screen *x11_screen,
@@ -1258,6 +1292,8 @@ gdk_x11_screen_supports_net_wm_hint (GdkX11Screen *x11_screen,
  * Returns: the name of the window manager screen @screen, or
  * "unknown" if the window manager is unknown. The string is owned by GDK
  * and should not be freed.
+ *
+ * Deprecated: 4.18
  **/
 const char*
 gdk_x11_screen_get_window_manager_name (GdkX11Screen *x11_screen)
@@ -1374,6 +1410,8 @@ get_netwm_cardinal_property (GdkX11Screen *x11_screen,
  * [Extended Window Manager Hints](http://www.freedesktop.org/Standards/wm-spec) specification.
  *
  * Returns: the number of workspaces, or 0 if workspaces are not supported
+ *
+ * Deprecated: 4.18
  */
 guint32
 gdk_x11_screen_get_number_of_desktops (GdkX11Screen *screen)
@@ -1391,6 +1429,8 @@ gdk_x11_screen_get_number_of_desktops (GdkX11Screen *screen)
  * [Extended Window Manager Hints](http://www.freedesktop.org/Standards/wm-spec) specification.
  *
  * Returns: the current workspace, or 0 if workspaces are not supported
+ *
+ * Deprecated: 4.18
  */
 guint32
 gdk_x11_screen_get_current_desktop (GdkX11Screen *screen)
