@@ -51,7 +51,10 @@ G_GNUC_BEGIN_IGNORE_DEPRECATIONS
  * Dialogs are a convenient way to prompt the user for a small amount
  * of input.
  *
- * ![An example GtkDialog](dialog.png)
+ * <picture>
+ *   <source srcset="dialog-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkDialog" src="dialog.png">
+ * </picture>
  *
  * Typical uses are to display a message, ask a question, or anything else
  * that does not require extensive effort on the user’s part.
@@ -454,7 +457,8 @@ gtk_dialog_constructed (GObject *object)
         }
       g_list_free (children);
 
-      _gtk_header_bar_track_default_decoration (GTK_HEADER_BAR (priv->headerbar));
+      if (GTK_IS_HEADER_BAR (priv->headerbar))
+        _gtk_header_bar_track_default_decoration (GTK_HEADER_BAR (priv->headerbar));
     }
   else
     {
@@ -1397,7 +1401,8 @@ gtk_dialog_buildable_add_child (GtkBuildable  *buildable,
   else if (g_str_equal (type, "titlebar"))
     {
       priv->headerbar = GTK_WIDGET (child);
-      _gtk_header_bar_track_default_decoration (GTK_HEADER_BAR (priv->headerbar));
+      if (GTK_IS_HEADER_BAR (priv->headerbar))
+        _gtk_header_bar_track_default_decoration (GTK_HEADER_BAR (priv->headerbar));
       gtk_window_set_titlebar (GTK_WINDOW (buildable), priv->headerbar);
     }
   else if (g_str_equal (type, "action"))
