@@ -612,6 +612,8 @@ gtk_reference_list_accessible_value_remove (GtkAccessibleValue *value,
   g_return_if_fail (value != NULL);
   g_return_if_fail (value->value_class == &GTK_REFERENCE_LIST_ACCESSIBLE_VALUE);
 
+  self->refs = g_list_remove (self->refs, ref);
+
   g_object_weak_unref (G_OBJECT (ref), remove_weak_ref_from_list, self);
 }
 
@@ -1403,7 +1405,7 @@ gtk_accessible_value_collect_value (const GtkAccessibleCollect  *cstate,
           }
         else 
           {
-            value = g_value_get_pointer (value_);
+            value = g_list_copy (g_value_get_pointer (value_));
           }
 
         if (ctor == NULL)
