@@ -53,10 +53,12 @@ static void     update_button_sensitivity
 /**
  * GtkColorDialogButton:
  *
- * The `GtkColorDialogButton` is a wrapped around a [class@Gtk.ColorDialog]
- * and allows to open a color chooser dialog to change the color.
+ * Opens a color chooser dialog to select a color.
  *
- * ![An example GtkColorDialogButton](color-button.png)
+ * <picture>
+ *   <source srcset="color-button-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkColorDialogButton" src="color-button.png">
+ * </picture>
  *
  * It is suitable widget for selecting a color in a preference dialog.
  *
@@ -155,6 +157,10 @@ gtk_color_dialog_button_init (GtkColorDialogButton *self)
   gtk_widget_add_css_class (self->button, "color");
 
   gtk_color_dialog_button_set_rgba (self, &(GdkRGBA) { 0.75, 0.25, 0.25, 1.0 });
+
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self->button),
+                                  GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, TRUE,
+                                  -1);
 }
 
 static void
@@ -257,7 +263,7 @@ gtk_color_dialog_button_class_init (GtkColorDialogButtonClass *class)
   widget_class->unroot = gtk_color_dialog_button_unroot;
 
   /**
-   * GtkColorDialogButton:dialog: (attributes org.gtk.Property.get=gtk_color_dialog_button_get_dialog org.gtk.Property.set=gtk_color_dialog_button_set_dialog)
+   * GtkColorDialogButton:dialog:
    *
    * The `GtkColorDialog` that contains parameters for
    * the color chooser dialog.
@@ -270,7 +276,7 @@ gtk_color_dialog_button_class_init (GtkColorDialogButtonClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkColorDialogButton:rgba: (attributes org.gtk.Property.get=gtk_color_dialog_button_get_rgba org.gtk.Property.set=gtk_color_dialog_button_set_rgba)
+   * GtkColorDialogButton:rgba:
    *
    * The selected color.
    *
@@ -560,4 +566,4 @@ gtk_color_dialog_button_set_rgba (GtkColorDialogButton *self,
 
 /* }}} */
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
