@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_CSS_SELECTOR_PRIVATE_H__
-#define __GTK_CSS_SELECTOR_PRIVATE_H__
+#pragma once
 
 #include <gtk/css/gtkcss.h>
 #include "gtk/css/gtkcsstokenizerprivate.h"
@@ -73,4 +72,3 @@ void                       _gtk_css_selector_tree_builder_free  (GtkCssSelectorT
 
 G_END_DECLS
 
-#endif /* __GTK_CSS_SELECTOR_PRIVATE_H__ */
