@@ -788,6 +788,7 @@ static const GtkCssValueClass GTK_CSS_VALUE_TRANSFORM = {
   "GtkCssTransformValue",
   gtk_css_value_transform_free,
   gtk_css_value_transform_compute,
+  NULL,
   gtk_css_value_transform_equal,
   gtk_css_value_transform_transition,
   NULL,
@@ -935,7 +936,9 @@ _gtk_css_transform_value_parse (GtkCssParser *parser)
 
   while (TRUE)
     {
-      GtkCssTransform transform = { 0, };
+      GtkCssTransform transform;
+
+      memset (&transform, 0, sizeof (GtkCssTransform));
 
       if (gtk_css_parser_has_function (parser, "matrix"))
         {
