@@ -448,6 +448,23 @@ gtk_list_base_select_item (GtkListBase *self,
                                       0, 0);
 }
 
+static void
+activate_listitem_select_action (GtkListBasePrivate *priv,
+                                 guint               pos,
+                                 gboolean            modify,
+                                 gboolean            extend)
+{
+  GtkListTile *tile;
+
+  tile = gtk_list_item_manager_get_nth (priv->item_manager, pos, NULL);
+
+  /* We do this convoluted calling into the widget because that way
+   * GtkListItem::selectable gets respected, which is what one would expect.
+   */
+  g_assert (tile->widget);
+  gtk_widget_activate_action (tile->widget, "listitem.select", "(bb)", modify, extend);
+}
+
 /*
  * gtk_list_base_grab_focus_on_item:
  * @self: a `GtkListBase`
@@ -506,13 +523,7 @@ gtk_list_base_grab_focus_on_item (GtkListBase *self,
 
   if (select)
     {
-      tile = gtk_list_item_manager_get_nth (priv->item_manager, pos, NULL);
-
-      /* We do this convoluted calling into the widget because that way
-       * GtkListItem::selectable gets respected, which is what one would expect.
-       */
-      g_assert (tile->widget);
-      gtk_widget_activate_action (tile->widget, "listitem.select", "(bb)", modify, extend);
+      activate_listitem_select_action (priv, pos, modify, extend);
     }
 
   return TRUE;
@@ -1131,6 +1142,35 @@ gtk_list_base_move_cursor_to_end (GtkWidget *widget,
   return TRUE;
 }
 
+static gboolean
+handle_selecting_unselected_cursor (GtkListBase *self,
+                                    guint        position,
+                                    gboolean     select,
+                                    gboolean     modify,
+                                    gboolean     extend)
+{
+  GtkListBasePrivate *priv = gtk_list_base_get_instance_private (self);
+  GtkSelectionModel *model;
+
+  /* If Ctrl is pressed, we don't want to reset the selection. */
+  if (!select || modify)
+    return FALSE;
+
+  model = gtk_list_item_manager_get_model (priv->item_manager);
+
+  /* Selection of current position is not needed if it's already selected or if
+   * there is nothing to select, or the position is invalid.
+   */
+  if (model == NULL || position == GTK_INVALID_LIST_POSITION || gtk_selection_model_is_selected (model, position))
+    return FALSE;
+
+  /* Reset cursor to current position trying to select it as well. */
+  activate_listitem_select_action (priv, position, FALSE, extend);
+
+  /* Report whether the model allowed the selection change. */
+  return gtk_selection_model_is_selected (model, position);
+}
+
 static gboolean
 gtk_list_base_move_cursor (GtkWidget *widget,
                            GVariant  *args,
@@ -1145,6 +1185,13 @@ gtk_list_base_move_cursor (GtkWidget *widget,
   g_variant_get (args, "(ubbbi)", &orientation, &select, &modify, &extend, &amount);
 
   old_pos = gtk_list_base_get_focus_position (self);
+
+  /* When the focus is on an unselected item while we're selecting, we want to
+   * not move focus but select the focused item instead if we can.
+   */
+  if (handle_selecting_unselected_cursor (self, old_pos, select, modify, extend))
+    return TRUE;
+
   new_pos = gtk_list_base_move_focus (self, old_pos, orientation, amount);
 
   if (old_pos != new_pos)
@@ -1500,8 +1547,8 @@ gtk_list_base_size_allocate_child (GtkListBase *self,
 
   if (!graphene_rect_intersection (gtk_css_boxes_get_padding_rect (boxes),
                                    &GRAPHENE_RECT_INIT(
-                                     child_allocation.x + GTK_LIST_BASE_CHILD_MAX_OVERDRAW,
-                                     child_allocation.y + GTK_LIST_BASE_CHILD_MAX_OVERDRAW,
+                                     child_allocation.x - GTK_LIST_BASE_CHILD_MAX_OVERDRAW,
+                                     child_allocation.y - GTK_LIST_BASE_CHILD_MAX_OVERDRAW,
                                      child_allocation.width + 2 * GTK_LIST_BASE_CHILD_MAX_OVERDRAW,
                                      child_allocation.height + 2 * GTK_LIST_BASE_CHILD_MAX_OVERDRAW
                                    ),
@@ -1707,7 +1754,7 @@ gtk_list_base_apply_rubberband_selection (GtkListBase *self,
   model = gtk_list_item_manager_get_model (priv->item_manager);
   if (model != NULL)
     {
-      GtkBitset *selected, *mask;
+      GtkBitset *selected, *mask, *result;
       GdkRectangle rect;
       GtkBitset *rubberband_selection;
 
@@ -1760,8 +1807,14 @@ gtk_list_base_apply_rubberband_selection (GtkListBase *self,
 
       gtk_selection_model_set_selection (model, selected, mask);
 
+      result = gtk_selection_model_get_selection (model);
+
+      if (gtk_bitset_get_size (result) == 1)
+        gtk_list_base_grab_focus_on_item (self, gtk_bitset_get_minimum (result), TRUE, FALSE, FALSE);
+
       gtk_bitset_unref (selected);
       gtk_bitset_unref (mask);
+      gtk_bitset_unref (result);
       gtk_bitset_unref (rubberband_selection);
     }
 }
@@ -2379,4 +2432,3 @@ gtk_list_base_scroll_to (GtkListBase        *self,
 
   gtk_list_base_scroll_to_item (self, pos, scroll);
 }
-
