@@ -21,9 +21,8 @@
 /*
  * GtkListListModel:
  *
- * #GtkListListModel is a #GListModel implementation that takes a list API
- * and provides it as a GListModel.
- **/
+ * A list model that takes a list API and provides it as a `GListModel`.
+ */
 
 #include "config.h"
 
@@ -41,6 +40,9 @@ struct _GtkListListModel
   gpointer (* get_item) (gpointer, gpointer);
   gpointer data;
   GDestroyNotify notify;
+
+  guint cache_pos;
+  gpointer cache_item;
 };
 
 struct _GtkListListModelClass
@@ -48,6 +50,16 @@ struct _GtkListListModelClass
   GObjectClass parent_class;
 };
 
+enum {
+  PROP_0,
+  PROP_ITEM_TYPE,
+  PROP_N_ITEMS,
+
+  N_PROPS
+};
+
+static GParamSpec *properties[N_PROPS] = { NULL, };
+
 static GType
 gtk_list_list_model_get_item_type (GListModel *list)
 {
@@ -62,6 +74,18 @@ gtk_list_list_model_get_n_items (GListModel *list)
   return self->n_items;
 }
 
+static gboolean
+gtk_list_list_model_cache_is_valid (GtkListListModel *self)
+{
+  return self->cache_item != NULL;
+}
+
+static void
+gtk_list_list_model_invalidate_cache (GtkListListModel *self)
+{
+  self->cache_item = NULL;
+}
+
 static gpointer
 gtk_list_list_model_get_item (GListModel *list,
                               guint       position)
@@ -69,31 +93,50 @@ gtk_list_list_model_get_item (GListModel *list,
   GtkListListModel *self = GTK_LIST_LIST_MODEL (list);
   gpointer result;
   guint i;
+  guint start, end;
 
   if (position >= self->n_items)
-    {
-      return NULL;
-    }
-  else if (self->get_last &&
-           position >= self->n_items / 2)
-    {
-      result = self->get_last (self->data);
+    return NULL;
 
-      for (i = self->n_items - 1; i > position; i--)
+  start = 0;
+  end = self->n_items;
+  if (gtk_list_list_model_cache_is_valid (self))
+    {
+      if (self->cache_pos <= position)
+        start = self->cache_pos;
+      else
+        end = self->cache_pos;
+    }
+
+  if (self->get_last &&
+      position > (start + end) / 2)
+    {
+      if (end == self->cache_pos && gtk_list_list_model_cache_is_valid (self))
+        result = self->get_previous (self->cache_item, self->data);
+      else
+        result = self->get_last (self->data);
+
+      for (i = end - 1; i > position; i--)
         {
           result = self->get_previous (result, self->data);
         }
     }
   else
     {
-      result = self->get_first (self->data);
+      if (start == self->cache_pos && gtk_list_list_model_cache_is_valid (self))
+        result = self->cache_item;
+      else
+        result = self->get_first (self->data);
 
-      for (i = 0; i < position; i++)
+      for (i = start; i < position; i++)
         {
           result = self->get_next (result, self->data);
         }
     }
 
+  self->cache_item = result;
+  self->cache_pos = position;
+
   return self->get_item (result, self->data);
 }
 
@@ -109,6 +152,30 @@ G_DEFINE_TYPE_WITH_CODE (GtkListListModel, gtk_list_list_model,
                          G_TYPE_OBJECT,
                          G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, gtk_list_list_model_list_model_init))
 
+static void
+gtk_list_list_model_get_property (GObject    *object,
+                                  guint       prop_id,
+                                  GValue     *value,
+                                  GParamSpec *pspec)
+{
+  GtkListListModel *self = GTK_LIST_LIST_MODEL (object);
+
+  switch (prop_id)
+    {
+    case PROP_ITEM_TYPE:
+      g_value_set_gtype (value, G_TYPE_OBJECT);
+      break;
+
+    case PROP_N_ITEMS:
+      g_value_set_uint (value, self->n_items);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+    }
+}
+
 static void
 gtk_list_list_model_dispose (GObject *object)
 {
@@ -128,7 +195,20 @@ gtk_list_list_model_class_init (GtkListListModelClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  object_class->get_property = gtk_list_list_model_get_property;
   object_class->dispose = gtk_list_list_model_dispose;
+
+  properties[PROP_ITEM_TYPE] =
+    g_param_spec_gtype ("item-type", NULL, NULL,
+                        G_TYPE_OBJECT,
+                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  properties[PROP_N_ITEMS] =
+    g_param_spec_uint ("n-items", NULL, NULL,
+                       0, G_MAXUINT, 0,
+                       G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
 }
 
 static void
@@ -228,9 +308,12 @@ gtk_list_list_model_item_added_at (GtkListListModel *self,
   g_return_if_fail (GTK_IS_LIST_LIST_MODEL (self));
   g_return_if_fail (position <= self->n_items);
 
-  self->n_items += 1;
+  self->n_items++;
+  if (position <= self->cache_pos)
+    self->cache_pos++;
 
   g_list_model_items_changed (G_LIST_MODEL (self), position, 0, 1);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_N_ITEMS]);
 }
 
 void
@@ -279,6 +362,12 @@ gtk_list_list_model_item_moved (GtkListListModel *self,
 
   min = MIN (position, previous_position);
   max = MAX (position, previous_position) + 1;
+
+  if (self->cache_item == item)
+    self->cache_pos = position;
+  else if (self->cache_pos >= min && self->cache_pos < max)
+    self->cache_pos += (self->cache_pos > position ? 1 : -1);
+
   g_list_model_items_changed (G_LIST_MODEL (self), min, max - min, max - min);
 }
 
@@ -290,8 +379,13 @@ gtk_list_list_model_item_removed_at (GtkListListModel *self,
   g_return_if_fail (position < self->n_items);
 
   self->n_items -= 1;
+  if (position == self->cache_pos)
+    gtk_list_list_model_invalidate_cache (self);
+  else if (position < self->cache_pos)
+    self->cache_pos--;
 
   g_list_model_items_changed (G_LIST_MODEL (self), position, 1, 0);
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_N_ITEMS]);
 }
 
 void
@@ -309,8 +403,13 @@ gtk_list_list_model_clear (GtkListListModel *self)
   self->n_items = 0;
   self->notify = NULL;
 
+  gtk_list_list_model_invalidate_cache (self);
+
   if (n_items > 0)
-    g_list_model_items_changed (G_LIST_MODEL (self), 0, n_items, 0);
+    {
+      g_list_model_items_changed (G_LIST_MODEL (self), 0, n_items, 0);
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_N_ITEMS]);
+    }
 }
 
 
