@@ -21,9 +21,12 @@
 /**
  * GtkMenuButton:
  *
- * The `GtkMenuButton` widget is used to display a popup when clicked.
+ * Displays a popup when clicked.
  *
- * ![An example GtkMenuButton](menu-button.png)
+ * <picture>
+ *   <source srcset="menu-button-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkMenuButton" src="menu-button.png">
+ * </picture>
  *
  * This popup can be provided either as a `GtkPopover` or as an abstract
  * `GMenuModel`.
@@ -82,7 +85,7 @@
  *
  * # Accessibility
  *
- * `GtkMenuButton` uses the %GTK_ACCESSIBLE_ROLE_BUTTON role.
+ * `GtkMenuButton` uses the [enum@Gtk.AccessibleRole.button] role.
  */
 
 #include "config.h"
@@ -106,6 +109,7 @@
 #include "gtkbuttonprivate.h"
 #include "gtknative.h"
 #include "gtkwindow.h"
+#include "gtkbuilderprivate.h"
 
 typedef struct _GtkMenuButtonClass   GtkMenuButtonClass;
 typedef struct _GtkMenuButtonPrivate GtkMenuButtonPrivate;
@@ -420,7 +424,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
   klass->activate = gtk_menu_button_activate;
 
   /**
-   * GtkMenuButton:menu-model: (attributes org.gtk.Property.get=gtk_menu_button_get_menu_model org.gtk.Property.set=gtk_menu_button_set_menu_model)
+   * GtkMenuButton:menu-model:
    *
    * The `GMenuModel` from which the popup will be created.
    *
@@ -433,7 +437,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE);
 
   /**
-   * GtkMenuButton:direction: (attributes org.gtk.Property.get=gtk_menu_button_get_direction org.gtk.Property.set=gtk_menu_button_set_direction)
+   * GtkMenuButton:direction:
    *
    * The `GtkArrowType` representing the direction in which the
    * menu or popover will be popped out.
@@ -445,7 +449,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:popover: (attributes org.gtk.Property.get=gtk_menu_button_get_popover org.gtk.Property.set=gtk_menu_button_set_popover)
+   * GtkMenuButton:popover:
    *
    * The `GtkPopover` that will be popped up when the button is clicked.
    */
@@ -455,7 +459,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            G_PARAM_READWRITE);
 
   /**
-   * GtkMenuButton:icon-name: (attributes org.gtk.Property.get=gtk_menu_button_get_icon_name org.gtk.Property.set=gtk_menu_button_set_icon_name)
+   * GtkMenuButton:icon-name:
    *
    * The name of the icon used to automatically populate the button.
    */
@@ -465,7 +469,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:always-show-arrow: (attributes org.gtk.Property.get=gtk_menu_button_get_always_show_arrow org.gtk.Property.set=gtk_menu_button_set_always_show_arrow)
+   * GtkMenuButton:always-show-arrow:
    *
    * Whether to show a dropdown arrow even when using an icon or a custom child.
    *
@@ -477,7 +481,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                             GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:label: (attributes org.gtk.Property.get=gtk_menu_button_get_label org.gtk.Property.set=gtk_menu_button_set_label)
+   * GtkMenuButton:label:
    *
    * The label for the button.
    */
@@ -487,7 +491,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:use-underline: (attributes org.gtk.Property.get=gtk_menu_button_get_use_underline org.gtk.Property.set=gtk_menu_button_set_use_underline)
+   * GtkMenuButton:use-underline:
    *
    * If set an underscore in the text indicates a mnemonic.
    */
@@ -497,7 +501,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                            GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:has-frame: (attributes org.gtk.Property.get=gtk_menu_button_get_has_frame org.gtk.Property.set=gtk_menu_button_set_has_frame)
+   * GtkMenuButton:has-frame:
    *
    * Whether the button has a frame.
    */
@@ -507,7 +511,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:primary: (attributes org.gtk.Property.get=gtk_menu_button_get_primary org.gtk.Property.set=gtk_menu_button_set_primary)
+   * GtkMenuButton:primary:
    *
    * Whether the menu button acts as a primary menu.
    *
@@ -521,7 +525,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:child: (attributes org.gtk.Property.get=gtk_menu_button_get_child org.gtk.Property.set=gtk_menu_button_set_child)
+   * GtkMenuButton:child:
    *
    * The child widget.
    *
@@ -533,7 +537,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                          GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:active: (attributes org.gtk.Property.get=gtk_menu_button_get_active org.gtk.Property.set=gtk_menu_button_set_active)
+   * GtkMenuButton:active:
    *
    * Whether the menu button is active.
    *
@@ -545,7 +549,7 @@ gtk_menu_button_class_init (GtkMenuButtonClass *klass)
                           GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkMenuButton:can-shrink: (attributes org.gtk.Property.get=gtk_menu_button_get_can_shrink org.gtk.Property.set=gtk_menu_button_set_can_shrink)
+   * GtkMenuButton:can-shrink:
    *
    * Whether the size of the button can be made smaller than the natural
    * size of its contents.
@@ -700,9 +704,14 @@ gtk_menu_button_buildable_add_child (GtkBuildable *buildable,
                                      const char   *type)
 {
   if (GTK_IS_WIDGET (child))
-    gtk_menu_button_set_child (GTK_MENU_BUTTON (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_menu_button_set_child (GTK_MENU_BUTTON (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -739,15 +748,15 @@ update_sensitivity (GtkMenuButton *self)
 
   gtk_widget_set_sensitive (self->button, has_popup);
 
-  gtk_accessible_update_property (GTK_ACCESSIBLE (self),
+  gtk_accessible_update_property (GTK_ACCESSIBLE (self->button),
                                   GTK_ACCESSIBLE_PROPERTY_HAS_POPUP, has_popup,
                                   -1);
   if (self->popover != NULL)
-    gtk_accessible_update_relation (GTK_ACCESSIBLE (self),
+    gtk_accessible_update_relation (GTK_ACCESSIBLE (self->button),
                                     GTK_ACCESSIBLE_RELATION_CONTROLS, self->popover, NULL,
                                     -1);
   else
-    gtk_accessible_reset_relation (GTK_ACCESSIBLE (self),
+    gtk_accessible_reset_relation (GTK_ACCESSIBLE (self->button),
                                    GTK_ACCESSIBLE_RELATION_CONTROLS);
 }
 
@@ -760,7 +769,7 @@ menu_deactivate_cb (GtkMenuButton *self)
 }
 
 /**
- * gtk_menu_button_set_menu_model: (attributes org.gtk.Method.set_property=menu-model)
+ * gtk_menu_button_set_menu_model:
  * @menu_button: a `GtkMenuButton`
  * @menu_model: (nullable): a `GMenuModel`, or %NULL to unset and disable the
  *   button
@@ -812,7 +821,7 @@ gtk_menu_button_set_menu_model (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_menu_model: (attributes org.gtk.Method.get_property=menu-model)
+ * gtk_menu_button_get_menu_model:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns the `GMenuModel` used to generate the popup.
@@ -860,7 +869,7 @@ popover_destroy_cb (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_direction: (attributes org.gtk.Method.set_property=direction)
+ * gtk_menu_button_set_direction:
  * @menu_button: a `GtkMenuButton`
  * @direction: a `GtkArrowType`
  *
@@ -892,7 +901,7 @@ gtk_menu_button_set_direction (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_direction: (attributes org.gtk.Method.get_property=direction)
+ * gtk_menu_button_get_direction:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns the direction the popup will be pointing at when popped up.
@@ -934,9 +943,10 @@ gtk_menu_button_dispose (GObject *object)
 }
 
 /**
- * gtk_menu_button_set_popover: (attributes org.gtk.Method.set_property=popover)
+ * gtk_menu_button_set_popover:
  * @menu_button: a `GtkMenuButton`
- * @popover: (nullable): a `GtkPopover`, or %NULL to unset and disable the button
+ * @popover: (nullable) (type GtkPopover): a `GtkPopover`, or %NULL to unset and
+ *   disable the button
  *
  * Sets the `GtkPopover` that will be popped up when the @menu_button is clicked.
  *
@@ -990,7 +1000,7 @@ gtk_menu_button_set_popover (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_popover: (attributes org.gtk.Method.get_property=popover)
+ * gtk_menu_button_get_popover:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns the `GtkPopover` that pops out of the button.
@@ -1009,7 +1019,7 @@ gtk_menu_button_get_popover (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_icon_name: (attributes org.gtk.Method.set_property=icon-name)
+ * gtk_menu_button_set_icon_name:
  * @menu_button: a `GtkMenuButton`
  * @icon_name: the icon name
  *
@@ -1065,7 +1075,7 @@ gtk_menu_button_set_icon_name (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_icon_name: (attributes org.gtk.Method.get_property=icon-name)
+ * gtk_menu_button_get_icon_name:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets the name of the icon shown in the button.
@@ -1084,7 +1094,7 @@ gtk_menu_button_get_icon_name (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_always_show_arrow: (attributes org.gtk.Method.set_property=always-show-arrow)
+ * gtk_menu_button_set_always_show_arrow:
  * @menu_button: a `GtkMenuButton`
  * @always_show_arrow: whether to show a dropdown arrow even when using an icon
  * or a custom child
@@ -1113,7 +1123,7 @@ gtk_menu_button_set_always_show_arrow (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_always_show_arrow: (attributes org.gtk.Method.get_property=always-show-arrow)
+ * gtk_menu_button_get_always_show_arrow:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets whether to show a dropdown arrow even when using an icon or a custom
@@ -1133,7 +1143,7 @@ gtk_menu_button_get_always_show_arrow (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_label: (attributes org.gtk.Method.set_property=label)
+ * gtk_menu_button_set_label:
  * @menu_button: a `GtkMenuButton`
  * @label: the label
  *
@@ -1189,7 +1199,7 @@ gtk_menu_button_set_label (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_label: (attributes org.gtk.Method.get_property=label)
+ * gtk_menu_button_get_label:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets the label shown in the button
@@ -1208,7 +1218,7 @@ gtk_menu_button_get_label (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_has_frame: (attributes org.gtk.Method.set_property=has-frame)
+ * gtk_menu_button_set_has_frame:
  * @menu_button: a `GtkMenuButton`
  * @has_frame: whether the button should have a visible frame
  *
@@ -1228,7 +1238,7 @@ gtk_menu_button_set_has_frame (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_has_frame: (attributes org.gtk.Method.get_property=has-frame)
+ * gtk_menu_button_get_has_frame:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether the button has a frame.
@@ -1312,7 +1322,7 @@ gtk_menu_button_set_create_popup_func (GtkMenuButton                *menu_button
 }
 
 /**
- * gtk_menu_button_set_use_underline: (attributes org.gtk.Method.set_property=use-underline)
+ * gtk_menu_button_set_use_underline:
  * @menu_button: a `GtkMenuButton`
  * @use_underline: %TRUE if underlines in the text indicate mnemonics
  *
@@ -1335,7 +1345,7 @@ gtk_menu_button_set_use_underline (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_use_underline: (attributes org.gtk.Method.get_property=use-underline)
+ * gtk_menu_button_get_use_underline:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether an embedded underline in the text indicates a
@@ -1410,7 +1420,7 @@ gtk_menu_button_unroot (GtkWidget *widget)
 }
 
 /**
- * gtk_menu_button_set_primary: (attributes org.gtk.Method.set_property=primary)
+ * gtk_menu_button_set_primary:
  * @menu_button: a `GtkMenuButton`
  * @primary: whether the menubutton should act as a primary menu
  *
@@ -1446,7 +1456,7 @@ gtk_menu_button_set_primary (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_primary: (attributes org.gtk.Method.get_property=primary)
+ * gtk_menu_button_get_primary:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether the menu button acts as a primary menu.
@@ -1464,7 +1474,7 @@ gtk_menu_button_get_primary (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_menu_button_set_child:
  * @menu_button: a `GtkMenuButton`
  * @child: (nullable): the child widget
  *
@@ -1535,7 +1545,7 @@ gtk_menu_button_set_child (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_menu_button_get_child:
  * @menu_button: a `GtkMenuButton`
  *
  * Gets the child widget of @menu_button.
@@ -1553,7 +1563,7 @@ gtk_menu_button_get_child (GtkMenuButton *menu_button)
 }
 
 /**
- * gtk_menu_button_set_active: (attributes org.gtk.Method.set_property=active)
+ * gtk_menu_button_set_active:
  * @menu_button: a `GtkMenuButton`
  * @active: whether the menu button is active
  *
@@ -1577,7 +1587,7 @@ gtk_menu_button_set_active (GtkMenuButton *menu_button,
 }
 
 /**
- * gtk_menu_button_get_active: (attributes org.gtk.Method.get_property=active)
+ * gtk_menu_button_get_active:
  * @menu_button: a `GtkMenuButton`
  *
  * Returns whether the menu button is active.
