@@ -173,7 +173,7 @@ gtk_menu_section_box_schedule_separator_sync (GtkMenuSectionBox *box)
 
   if (!box->separator_sync_idle)
     {
-      box->separator_sync_idle = g_idle_add_full (G_PRIORITY_HIGH_IDLE, /* before resize... */
+      box->separator_sync_idle = g_idle_add_full (G_PRIORITY_DEFAULT, /* before menu is drawn... */
                                                   gtk_menu_section_box_handle_sync_separators,
                                                   box, NULL);
       gdk_source_set_static_name_by_id (box->separator_sync_idle, "[gtk] menu section box handle sync separators");
@@ -359,6 +359,7 @@ gtk_menu_section_box_insert_func (GtkMenuTrackerItem *item,
                                  NULL);
           g_object_bind_property (item, "label", widget, "text", G_BINDING_SYNC_CREATE);
           g_object_bind_property (item, "icon", widget, "icon", G_BINDING_SYNC_CREATE);
+          g_object_bind_property (item, "use-markup", widget, "use-markup", G_BINDING_SYNC_CREATE);
           g_object_bind_property (item, "sensitive", widget, "sensitive", G_BINDING_SYNC_CREATE);
 
           g_signal_connect (submenu, "show", G_CALLBACK (submenu_shown), item);
@@ -376,6 +377,7 @@ gtk_menu_section_box_insert_func (GtkMenuTrackerItem *item,
                                  NULL);
           g_object_bind_property (item, "label", widget, "text", G_BINDING_SYNC_CREATE);
           g_object_bind_property (item, "icon", widget, "icon", G_BINDING_SYNC_CREATE);
+          g_object_bind_property (item, "use-markup", widget, "use-markup", G_BINDING_SYNC_CREATE);
           g_object_bind_property (item, "sensitive", widget, "sensitive", G_BINDING_SYNC_CREATE);
 
           get_ancestors (GTK_WIDGET (box->toplevel), GTK_TYPE_STACK, &stack, &parent);
@@ -572,7 +574,7 @@ gtk_menu_section_box_new_toplevel (GtkPopoverMenu      *popover,
                                        gtk_menu_section_box_insert_func,
                                        gtk_menu_section_box_remove_func, box);
 
-  g_signal_connect (G_OBJECT (popover), "notify::position", G_CALLBACK (update_popover_position_cb), box);
+  g_signal_connect_object (G_OBJECT (popover), "notify::position", G_CALLBACK (update_popover_position_cb), box, G_CONNECT_DEFAULT);
 }
 
 static void
@@ -677,6 +679,10 @@ gtk_menu_section_box_new_section (GtkMenuTrackerItem *item,
           gtk_widget_set_halign (title, GTK_ALIGN_START);
           g_object_bind_property (item, "label", title, "label", G_BINDING_SYNC_CREATE);
           gtk_box_append (GTK_BOX (box->item_box), title);
+          gtk_accessible_update_relation (GTK_ACCESSIBLE (box),
+                                          GTK_ACCESSIBLE_RELATION_LABELLED_BY,
+                                          title,
+                                          NULL, -1);
         }
 
       item_box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
@@ -711,6 +717,10 @@ gtk_menu_section_box_new_section (GtkMenuTrackerItem *item,
       gtk_label_set_xalign (GTK_LABEL (title), 0.0);
       gtk_widget_add_css_class (title, "title");
       gtk_box_append (GTK_BOX (box->separator), title);
+      gtk_accessible_update_relation (GTK_ACCESSIBLE (box),
+                                      GTK_ACCESSIBLE_RELATION_LABELLED_BY,
+                                      title,
+                                      NULL, -1);
     }
   else
     {
