@@ -1,5 +1,4 @@
-#ifndef __GTK_NATIVE_PRIVATE_H__
-#define __GTK_NATIVE_PRIVATE_H__
+#pragma once
 
 #include "gtknative.h"
 
@@ -8,7 +7,7 @@ G_BEGIN_DECLS
 /**
  * GtkNativeIface:
  *
- * The list of functions that must be implemented for the #GtkNative interface.
+ * The list of functions that must be implemented for the `GtkNative` interface.
  */
 struct _GtkNativeInterface
 {
@@ -32,4 +31,3 @@ void    gtk_native_queue_relayout         (GtkNative    *native);
 
 G_END_DECLS
 
-#endif /* __GTK_NATIVE_PRIVATE_H__ */
