@@ -41,10 +41,12 @@
 /**
  * GtkPopoverMenu:
  *
- * `GtkPopoverMenu` is a subclass of `GtkPopover` that implements menu
- * behavior.
+ * A subclass of `GtkPopover` that implements menu behavior.
  *
- * ![An example GtkPopoverMenu](menu.png)
+ * <picture>
+ *   <source srcset="menu-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkPopoverMenu" src="menu.png">
+ * </picture>
  *
  * `GtkPopoverMenu` treats its children like menus and allows switching
  * between them. It can open submenus as traditional, nested submenus,
@@ -125,6 +127,8 @@
  *
  * - "label": a user-visible string to display
  * - "icon": icon name to display
+ * - "gtk-macos-special": (macOS only, ignored by others) Add special meaning to a menu
+ *     in the macOS menu bar. See [Using GTK on Apple macOS](osx.html).
  *
  * Menu items will also show accelerators, which are usually associated
  * with actions via [method@Gtk.Application.set_accels_for_action],
@@ -152,11 +156,10 @@
  *
  * # Accessibility
  *
- * `GtkPopoverMenu` uses the %GTK_ACCESSIBLE_ROLE_MENU role, and its
- * items use the %GTK_ACCESSIBLE_ROLE_MENU_ITEM,
- * %GTK_ACCESSIBLE_ROLE_MENU_ITEM_CHECKBOX or
- * %GTK_ACCESSIBLE_ROLE_MENU_ITEM_RADIO roles, depending on the
- * action they are connected to.
+ * `GtkPopoverMenu` uses the [enum@Gtk.AccessibleRole.menu] role, and its
+ * items use the [enum@Gtk.AccessibleRole.menu_item],
+ * [enum@Gtk.AccessibleRole.checkbox] or [enum@Gtk.AccessibleRole.menu_item_radio]
+ * roles, depending on the action they are connected to.
  */
 
 typedef struct _GtkPopoverMenuClass GtkPopoverMenuClass;
@@ -322,7 +325,7 @@ gtk_popover_menu_init (GtkPopoverMenu *popover)
   guint n_controllers, i;
 
   sw = gtk_scrolled_window_new ();
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_scrolled_window_set_propagate_natural_width (GTK_SCROLLED_WINDOW (sw), TRUE);
   gtk_scrolled_window_set_propagate_natural_height (GTK_SCROLLED_WINDOW (sw), TRUE);
   gtk_popover_set_child (GTK_POPOVER (popover), sw);
@@ -355,7 +358,6 @@ gtk_popover_menu_init (GtkPopoverMenu *popover)
     }
   g_free (controllers);
 
-  gtk_popover_disable_auto_mnemonics (GTK_POPOVER (popover));
   gtk_popover_set_cascade_popdown (GTK_POPOVER (popover), TRUE);
 }
 
@@ -620,7 +622,7 @@ gtk_popover_menu_class_init (GtkPopoverMenuClass *klass)
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GtkPopoverMenu:menu-model: (attributes org.gtk.Property.get=gtk_popover_menu_get_menu_model org.gtk.Property.set=gtk_popover_menu_set_menu_model)
+   * GtkPopoverMenu:menu-model:
    *
    * The model from which the menu is made.
    */
@@ -631,7 +633,7 @@ gtk_popover_menu_class_init (GtkPopoverMenuClass *klass)
                                                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   /**
-   * GtkPopoverMenu:flags: (attributes org.gtk.Property.get=gtk_popover_menu_get_flags org.gtk.Property.set=gtk_popover_menu_set_flags)
+   * GtkPopoverMenu:flags:
    *
    * The flags that @popover uses to create/display a menu from its model.
    *
@@ -824,7 +826,7 @@ gtk_popover_menu_new_from_model_full (GMenuModel          *model,
 }
 
 /**
- * gtk_popover_menu_set_menu_model: (attributes org.gtk.Method.set_property=menu-model)
+ * gtk_popover_menu_set_menu_model:
  * @popover: a `GtkPopoverMenu`
  * @model: (nullable): a `GMenuModel`
  *
@@ -849,7 +851,7 @@ gtk_popover_menu_set_menu_model (GtkPopoverMenu *popover,
 }
 
 /**
- * gtk_popover_menu_set_flags: (attributes org.gtk.Method.set_property=flags)
+ * gtk_popover_menu_set_flags:
  * @popover: a `GtkPopoverMenu`
  * @flags: a set of `GtkPopoverMenuFlags`
  *
@@ -879,7 +881,7 @@ gtk_popover_menu_set_flags (GtkPopoverMenu      *popover,
 }
 
 /**
- * gtk_popover_menu_get_menu_model: (attributes org.gtk.Method.get_property=menu-model)
+ * gtk_popover_menu_get_menu_model:
  * @popover: a `GtkPopoverMenu`
  *
  * Returns the menu model used to populate the popover.
@@ -895,7 +897,7 @@ gtk_popover_menu_get_menu_model (GtkPopoverMenu *popover)
 }
 
 /**
- * gtk_popover_menu_get_flags: (attributes org.gtk.Method.get_property=flags)
+ * gtk_popover_menu_get_flags:
  * @popover: a `GtkPopoverMenu`
  *
  * Returns the flags that @popover uses to create/display a menu from its model.
@@ -944,7 +946,7 @@ gtk_popover_menu_add_child (GtkPopoverMenu *popover,
  * @child: the `GtkWidget` to remove
  *
  * Removes a widget that has previously been added with
- * [method@Gtk.PopoverMenu.add_child()]
+ * [method@Gtk.PopoverMenu.add_child]
  *
  * Returns: %TRUE if the widget was removed
  */
