@@ -31,6 +31,7 @@
 
 #include "gtkbinlayout.h"
 #include "gtkbuildable.h"
+#include "gtkbuilderprivate.h"
 #include "gtkbutton.h"
 #include "gtkcenterbox.h"
 #include "gtkentryprivate.h"
@@ -44,9 +45,12 @@
 /**
  * GtkSearchBar:
  *
- * `GtkSearchBar` is a container made to have a search entry.
+ * Reveals a search entry when search is started.
  *
- * ![An example GtkSearchBar](search-bar.png)
+ * <picture>
+ *   <source srcset="search-bar-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkSearchBar" src="search-bar.png">
+ * </picture>
  *
  * It can also contain additional widgets, such as drop-down menus,
  * or buttons.  The search bar would appear when a search is started
@@ -93,7 +97,7 @@
  *
  * # Accessibility
  *
- * `GtkSearchBar` uses the %GTK_ACCESSIBLE_ROLE_SEARCH role.
+ * `GtkSearchBar` uses the [enum@Gtk.AccessibleRole.search] role.
  */
 
 typedef struct _GtkSearchBarClass   GtkSearchBarClass;
@@ -145,9 +149,14 @@ gtk_search_bar_buildable_add_child (GtkBuildable *buildable,
                                     const char   *type)
 {
   if (GTK_IS_WIDGET (child))
-    gtk_search_bar_set_child (GTK_SEARCH_BAR (buildable), GTK_WIDGET (child));
+    {
+      gtk_buildable_child_deprecation_warning (buildable, builder, NULL, "child");
+      gtk_search_bar_set_child (GTK_SEARCH_BAR (buildable), GTK_WIDGET (child));
+    }
   else
-    parent_buildable_iface->add_child (buildable, builder, child, type);
+    {
+      parent_buildable_iface->add_child (buildable, builder, child, type);
+    }
 }
 
 static void
@@ -308,7 +317,7 @@ gtk_search_bar_class_init (GtkSearchBarClass *klass)
   widget_class->focus = gtk_widget_focus_child;
 
   /**
-   * GtkSearchBar:search-mode-enabled: (attributes org.gtk.Property.get=gtk_search_bar_get_search_mode org.gtk.Property.set=gtk_search_bar_set_search_mode)
+   * GtkSearchBar:search-mode-enabled: (getter get_search_mode) (setter set_search_mode)
    *
    * Whether the search mode is on and the search bar shown.
    */
@@ -317,7 +326,7 @@ gtk_search_bar_class_init (GtkSearchBarClass *klass)
                                                                  GTK_PARAM_READWRITE|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkSearchBar:show-close-button: (attributes org.gtk.Property.get=gtk_search_bar_get_show_close_button org.gtk.Property.set=gtk_search_bar_set_show_close_button)
+   * GtkSearchBar:show-close-button:
    *
    * Whether to show the close button in the search bar.
    */
@@ -326,7 +335,7 @@ gtk_search_bar_class_init (GtkSearchBarClass *klass)
                                                                GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkSearchBar:child: (attributes org.gtk.Property.get=gtk_search_bar_get_child org.gtk.Property.set=gtk_search_bar_set_child)
+   * GtkSearchBar:child:
    *
    * The child widget.
    */
@@ -335,7 +344,7 @@ gtk_search_bar_class_init (GtkSearchBarClass *klass)
                                                   GTK_PARAM_READWRITE|G_PARAM_CONSTRUCT|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkSearchBar:key-capture-widget: (attributes org.gtk.Property.get=gtk_search_bar_get_key_capture_widget org.gtk.Property.set=gtk_search_bar_set_key_capture_widget)
+   * GtkSearchBar:key-capture-widget:
    *
    * The key capture widget.
    */
@@ -446,7 +455,7 @@ gtk_search_bar_connect_entry (GtkSearchBar *bar,
 }
 
 /**
- * gtk_search_bar_get_search_mode: (attributes org.gtk.Method.get_property=search-mode-enabled)
+ * gtk_search_bar_get_search_mode: (get-property search-mode-enabled)
  * @bar: a `GtkSearchBar`
  *
  * Returns whether the search mode is on or off.
@@ -462,7 +471,7 @@ gtk_search_bar_get_search_mode (GtkSearchBar *bar)
 }
 
 /**
- * gtk_search_bar_set_search_mode: (attributes org.gtk.Method.set_property=search-mode-enabled)
+ * gtk_search_bar_set_search_mode: (set-property search-mode-enabled)
  * @bar: a `GtkSearchBar`
  * @search_mode: the new state of the search mode
  *
@@ -478,7 +487,7 @@ gtk_search_bar_set_search_mode (GtkSearchBar *bar,
 }
 
 /**
- * gtk_search_bar_get_show_close_button: (attributes org.gtk.Method.get_property=show-close-button)
+ * gtk_search_bar_get_show_close_button:
  * @bar: a `GtkSearchBar`
  *
  * Returns whether the close button is shown.
@@ -494,7 +503,7 @@ gtk_search_bar_get_show_close_button (GtkSearchBar *bar)
 }
 
 /**
- * gtk_search_bar_set_show_close_button: (attributes org.gtk.Method.set_property=show-close-button)
+ * gtk_search_bar_set_show_close_button:
  * @bar: a `GtkSearchBar`
  * @visible: whether the close button will be shown or not
  *
@@ -599,7 +608,7 @@ capture_widget_key_handled (GtkEventControllerKey *controller,
 }
 
 /**
- * gtk_search_bar_set_key_capture_widget: (attributes org.gtk.Method.set_property=key-capture-widget)
+ * gtk_search_bar_set_key_capture_widget:
  * @bar: a `GtkSearchBar`
  * @widget: (nullable) (transfer none): a `GtkWidget`
  *
@@ -642,6 +651,8 @@ gtk_search_bar_set_key_capture_widget (GtkSearchBar *bar,
                                  (gpointer *) &bar->capture_widget);
 
       bar->capture_widget_controller = gtk_event_controller_key_new ();
+      gtk_event_controller_set_static_name (bar->capture_widget_controller,
+                                            "gtk-search-bar-capture");
       gtk_event_controller_set_propagation_phase (bar->capture_widget_controller,
                                                   GTK_PHASE_BUBBLE);
       g_signal_connect (bar->capture_widget_controller, "key-pressed",
@@ -655,7 +666,7 @@ gtk_search_bar_set_key_capture_widget (GtkSearchBar *bar,
 }
 
 /**
- * gtk_search_bar_get_key_capture_widget: (attributes org.gtk.Method.get_property=key-capture-widget)
+ * gtk_search_bar_get_key_capture_widget:
  * @bar: a `GtkSearchBar`
  *
  * Gets the widget that @bar is capturing key events from.
@@ -671,7 +682,7 @@ gtk_search_bar_get_key_capture_widget (GtkSearchBar *bar)
 }
 
 /**
- * gtk_search_bar_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_search_bar_set_child:
  * @bar: a `GtkSearchBar`
  * @child: (nullable): the child widget
  *
@@ -709,7 +720,7 @@ gtk_search_bar_set_child (GtkSearchBar *bar,
 }
 
 /**
- * gtk_search_bar_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_search_bar_get_child:
  * @bar: a `GtkSearchBar`
  *
  * Gets the child widget of @bar.
