@@ -124,14 +124,6 @@ struct _GtkTextLayout
   PangoContext *ltr_context;
   PangoContext *rtl_context;
 
-  /* A cache of one style; this is used to ensure
-   * we don't constantly regenerate the style
-   * over long runs with the same style. */
-  GtkTextAttributes *one_style_cache;
-
-  /* Whether we are allowed to wrap right now */
-  int wrap_loop_count;
-
   /* Whether to show the insertion cursor */
   guint cursor_visible : 1;
 
@@ -140,10 +132,10 @@ struct _GtkTextLayout
    */
   guint cursor_direction : 2;
 
-  /* The keyboard direction is used to default the alignment when
+  /* The default direction is used for alignment when
      there are no strong characters.
   */
-  guint keyboard_direction : 2;
+  guint default_direction : 2;
 
   guint overwrite_mode : 1;
 
@@ -235,8 +227,8 @@ void               gtk_text_layout_set_cursor_direction  (GtkTextLayout     *lay
                                                           GtkTextDirection   direction);
 void		   gtk_text_layout_set_overwrite_mode	 (GtkTextLayout     *layout,
 							  gboolean           overwrite);
-void               gtk_text_layout_set_keyboard_direction (GtkTextLayout     *layout,
-							   GtkTextDirection keyboard_dir);
+void               gtk_text_layout_set_default_direction (GtkTextLayout     *layout,
+							   GtkTextDirection default_dir);
 void               gtk_text_layout_default_style_changed (GtkTextLayout     *layout);
 
 void gtk_text_layout_set_screen_width       (GtkTextLayout     *layout,
@@ -372,11 +364,12 @@ void gtk_text_anchored_child_set_layout     (GtkWidget          *child,
 
 void gtk_text_layout_spew (GtkTextLayout *layout);
 
-void gtk_text_layout_snapshot (GtkTextLayout        *layout,
-                               GtkWidget            *widget,
-                               GtkSnapshot          *snapshot,
-                               const GdkRectangle   *clip,
-                               float                 cursor_alpha);
+void gtk_text_layout_snapshot (GtkTextLayout         *layout,
+                               GtkWidget             *widget,
+                               GtkSnapshot           *snapshot,
+                               const graphene_rect_t *clip,
+                               gboolean               selection_style_changed,
+                               float                  cursor_alpha);
 
 void gtk_text_layout_set_mru_size (GtkTextLayout *layout,
                                    guint          mru_size);
