@@ -22,11 +22,9 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
  */
 
-#ifndef __GTK_TOOLTIP_WINDOW_PRIVATE_H__
-#define __GTK_TOOLTIP_WINDOW_PRIVATE_H__
+#pragma once
 
 #include <gio/gio.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtkwindow.h>
 
 G_BEGIN_DECLS
@@ -37,6 +35,8 @@ G_DECLARE_FINAL_TYPE (GtkTooltipWindow, gtk_tooltip_window, GTK, TOOLTIP_WINDOW,
 
 GtkWidget *     gtk_tooltip_window_new                          (void);
 
+void            gtk_tooltip_window_present                      (GtkTooltipWindow *window);
+
 void            gtk_tooltip_window_set_label_markup             (GtkTooltipWindow *window,
                                                                  const char       *markup);
 void            gtk_tooltip_window_set_label_text               (GtkTooltipWindow *window,
@@ -61,4 +61,3 @@ void            gtk_tooltip_window_position                     (GtkTooltipWindo
 
 G_END_DECLS
 
-#endif /* __GTK_TOOLTIP_WINDOW_PRIVATE_H__ */
