@@ -48,6 +48,7 @@
 #include <gsk/gskrendernodeprivate.h>
 #include <gsk/gskroundedrectprivate.h>
 #include <gsk/gsktransformprivate.h>
+#include <gsk/gskcomponenttransferprivate.h>
 
 #include <cairo-gobject.h>
 #include <glib/gi18n-lib.h>
@@ -224,6 +225,7 @@ struct _GtkInspectorRecorder
   gboolean highlight_sequences;
   gboolean record_events;
   gboolean stop_after_next_frame;
+  gboolean save_to_file;
   gboolean dark;
 
   GdkEventSequence *selected_sequence;
@@ -403,6 +405,9 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
     case GSK_SUBSURFACE_NODE:
       return create_render_node_list_model (&(RenderNode) { gsk_subsurface_node_get_child (node), NULL }, 1);
+
+    case GSK_COMPONENT_TRANSFER_NODE:
+      return create_render_node_list_model (&(RenderNode) { gsk_component_transfer_node_get_child (node), NULL }, 1);
     }
 }
 
@@ -491,6 +496,8 @@ node_type_name (GskRenderNodeType type)
       return "GL Shader";
     case GSK_SUBSURFACE_NODE:
       return "Subsurface";
+    case GSK_COMPONENT_TRANSFER_NODE:
+      return "Component Transfer";
     }
 }
 
@@ -528,13 +535,14 @@ node_name (GskRenderNode *node)
     case GSK_BLUR_NODE:
     case GSK_GL_SHADER_NODE:
     case GSK_SUBSURFACE_NODE:
+    case GSK_COMPONENT_TRANSFER_NODE:
       return g_strdup (node_type_name (gsk_render_node_get_node_type (node)));
 
     case GSK_DEBUG_NODE:
       return g_strdup (gsk_debug_node_get_message (node));
 
     case GSK_COLOR_NODE:
-      return gdk_color_to_string (gsk_color_node_get_color2 (node));
+      return gdk_color_to_string (gsk_color_node_get_gdk_color (node));
 
     case GSK_TEXTURE_NODE:
       {
@@ -898,7 +906,7 @@ get_color2_texture (const GdkColor *color)
 }
 
 static GdkTexture *
-get_linear_gradient_texture (gsize n_stops, const GskColorStop2 *stops)
+get_linear_gradient_texture (gsize n_stops, const GskGradientStop *stops)
 {
   cairo_surface_t *surface;
   cairo_t *cr;
@@ -1196,7 +1204,7 @@ populate_render_node_properties (GListStore    *store,
       break;
 
     case GSK_COLOR_NODE:
-      add_color_row (store, "Color", gsk_color_node_get_color2 (node));
+      add_color_row (store, "Color", gsk_color_node_get_gdk_color (node));
       break;
 
     case GSK_LINEAR_GRADIENT_NODE:
@@ -1204,10 +1212,10 @@ populate_render_node_properties (GListStore    *store,
       {
         const graphene_point_t *start = gsk_linear_gradient_node_get_start (node);
         const graphene_point_t *end = gsk_linear_gradient_node_get_end (node);
-        const gsize n_stops = gsk_linear_gradient_node_get_n_color_stops (node);
-        const GskColorStop2 *stops = gsk_linear_gradient_node_get_color_stops2 (node);
-        GdkColorState *interpolation = gsk_linear_gradient_node_get_interpolation_color_state (node);
-        GskHueInterpolation hue_interpolation = gsk_linear_gradient_node_get_hue_interpolation (node);
+        const gsize n_stops = gsk_gradient_node_get_n_stops (node);
+        const GskGradientStop *stops = gsk_gradient_node_get_stops (node);
+        GdkColorState *interpolation = gsk_gradient_node_get_interpolation (node);
+        GskHueInterpolation hue_interpolation = gsk_gradient_node_get_hue_interpolation (node);
         int i;
         GString *s;
         GdkTexture *texture;
@@ -1219,6 +1227,8 @@ populate_render_node_properties (GListStore    *store,
         s = g_string_new ("");
         for (i = 0; i < n_stops; i++)
           {
+            if (i > 0 && stops[i].transition_hint != 0.5)
+              g_string_append_printf (s, "⟝ %.2f ⟞\n", stops[i].transition_hint);
             g_string_append_printf (s, "%.2f, ", stops[i].offset);
             gdk_color_print (&stops[i].color, s);
             g_string_append_c (s, '\n');
@@ -1240,10 +1250,10 @@ populate_render_node_properties (GListStore    *store,
         const float end = gsk_radial_gradient_node_get_end (node);
         const float hradius = gsk_radial_gradient_node_get_hradius (node);
         const float vradius = gsk_radial_gradient_node_get_vradius (node);
-        const gsize n_stops = gsk_radial_gradient_node_get_n_color_stops (node);
-        const GskColorStop2 *stops = gsk_radial_gradient_node_get_color_stops2 (node);
-        GdkColorState *interpolation = gsk_radial_gradient_node_get_interpolation_color_state (node);
-        GskHueInterpolation hue_interpolation = gsk_radial_gradient_node_get_hue_interpolation (node);
+        const gsize n_stops = gsk_gradient_node_get_n_stops (node);
+        const GskGradientStop *stops = gsk_gradient_node_get_stops (node);
+        GdkColorState *interpolation = gsk_gradient_node_get_interpolation (node);
+        GskHueInterpolation hue_interpolation = gsk_gradient_node_get_hue_interpolation (node);
         int i;
         GString *s;
         GdkTexture *texture;
@@ -1257,6 +1267,8 @@ populate_render_node_properties (GListStore    *store,
         s = g_string_new ("");
         for (i = 0; i < n_stops; i++)
           {
+            if (i > 0 && stops[i].transition_hint != 0.5)
+              g_string_append_printf (s, "⟝ %.2f ⟞\n", stops[i].transition_hint);
             g_string_append_printf (s, "%.2f, ", stops[i].offset);
             gdk_color_print (&stops[i].color, s);
             g_string_append_c (s, '\n');
@@ -1274,10 +1286,10 @@ populate_render_node_properties (GListStore    *store,
       {
         const graphene_point_t *center = gsk_conic_gradient_node_get_center (node);
         const float rotation = gsk_conic_gradient_node_get_rotation (node);
-        const gsize n_stops = gsk_conic_gradient_node_get_n_color_stops (node);
-        const GskColorStop2 *stops = gsk_conic_gradient_node_get_color_stops2 (node);
-        GdkColorState *interpolation = gsk_conic_gradient_node_get_interpolation_color_state (node);
-        GskHueInterpolation hue_interpolation = gsk_conic_gradient_node_get_hue_interpolation (node);
+        const gsize n_stops = gsk_gradient_node_get_n_stops (node);
+        const GskGradientStop *stops = gsk_gradient_node_get_stops (node);
+        GdkColorState *interpolation = gsk_gradient_node_get_interpolation (node);
+        GskHueInterpolation hue_interpolation = gsk_gradient_node_get_hue_interpolation (node);
         gsize i;
         GString *s;
         GdkTexture *texture;
@@ -1290,6 +1302,8 @@ populate_render_node_properties (GListStore    *store,
         s = g_string_new ("");
         for (i = 0; i < n_stops; i++)
           {
+            if (i > 0 && stops[i].transition_hint != 0.5)
+              g_string_append_printf (s, "⟝ %.2f ⟞\n", stops[i].transition_hint);
             g_string_append_printf (s, "%.2f, ", stops[i].offset);
             gdk_color_print (&stops[i].color, s);
             g_string_append_c (s, '\n');
@@ -1341,7 +1355,7 @@ populate_render_node_properties (GListStore    *store,
 
         add_text_row (store, "Position", "%.2f %.2f", offset->x, offset->y);
 
-        add_color_row (store, "Color", gsk_text_node_get_color2 (node));
+        add_color_row (store, "Color", gsk_text_node_get_gdk_color (node));
       }
       break;
 
@@ -1349,7 +1363,7 @@ populate_render_node_properties (GListStore    *store,
       {
         const char *name[4] = { "Top", "Right", "Bottom", "Left" };
         const float *widths = gsk_border_node_get_widths (node);
-        const GdkColor *colors = gsk_border_node_get_colors2 (node);
+        const GdkColor *colors = gsk_border_node_get_gdk_colors (node);
         int i;
 
         for (i = 0; i < 4; i++)
@@ -1481,7 +1495,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
 
     case GSK_INSET_SHADOW_NODE:
       {
-        const GdkColor *color = gsk_inset_shadow_node_get_color2 (node);
+        const GdkColor *color = gsk_inset_shadow_node_get_gdk_color (node);
         float dx = gsk_inset_shadow_node_get_dx (node);
         float dy = gsk_inset_shadow_node_get_dy (node);
         float spread = gsk_inset_shadow_node_get_spread (node);
@@ -1499,7 +1513,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
     case GSK_OUTSET_SHADOW_NODE:
       {
         const GskRoundedRect *outline = gsk_outset_shadow_node_get_outline (node);
-        const GdkColor *color = gsk_outset_shadow_node_get_color2 (node);
+        const GdkColor *color = gsk_outset_shadow_node_get_gdk_color (node);
         float dx = gsk_outset_shadow_node_get_dx (node);
         float dy = gsk_outset_shadow_node_get_dy (node);
         float spread = gsk_outset_shadow_node_get_spread (node);
@@ -1644,7 +1658,7 @@ G_GNUC_END_IGNORE_DEPRECATIONS
         for (i = 0; i < gsk_shadow_node_get_n_shadows (node); i++)
           {
             char *label;
-            const GskShadow2 *shadow = gsk_shadow_node_get_shadow2 (node, i);
+            const GskShadowEntry *shadow = gsk_shadow_node_get_shadow_entry (node, i);
 
             label = g_strdup_printf ("Color %d", i);
             add_color_row (store, label, &shadow->color);
@@ -1693,6 +1707,22 @@ G_GNUC_END_IGNORE_DEPRECATIONS
       }
       break;
 
+    case GSK_COMPONENT_TRANSFER_NODE:
+      {
+        const char *component[] = { "Red", "Green", "Blue", "Alpha" };
+        GString *s = g_string_new ("");
+
+        for (guint i = 0; i < 4; i++)
+          {
+            g_string_set_size (s, 0);
+            gsk_component_transfer_print (gsk_component_transfer_node_get_transfer (node, i), s);
+            add_text_row (store, component[i], "%s", s->str);
+          }
+
+        g_string_free (s, TRUE);
+      }
+      break;
+
     case GSK_NOT_A_RENDER_NODE:
     default:
       break;
@@ -1732,6 +1762,7 @@ event_type_name (GdkEventType type)
     "Pad Strip",
     "Pad Group Mode",
     "Touchpad Hold",
+    "Pad Dial",
   };
 
   G_STATIC_ASSERT (G_N_ELEMENTS (event_name) == GDK_EVENT_LAST);
@@ -2315,6 +2346,7 @@ get_event_summary (GdkEvent *event)
     case GDK_PAD_BUTTON_RELEASE:
     case GDK_PAD_RING:
     case GDK_PAD_STRIP:
+    case GDK_PAD_DIAL:
     case GDK_PAD_GROUP_MODE:
     case GDK_DELETE:
       return g_strdup_printf ("%s", name);
@@ -2688,7 +2720,8 @@ gtk_inspector_recorder_set_recording (GtkInspectorRecorder *recorder,
 }
 
 void
-gtk_inspector_recorder_record_single_frame (GtkInspectorRecorder *recorder)
+gtk_inspector_recorder_record_single_frame (GtkInspectorRecorder *recorder,
+                                            gboolean              save_to_file)
 {
   if (gtk_inspector_recorder_is_recording (recorder))
     return;
@@ -2697,6 +2730,7 @@ gtk_inspector_recorder_record_single_frame (GtkInspectorRecorder *recorder)
   recorder->start_time = 0;
   recorder->record_events = FALSE;
   recorder->stop_after_next_frame = TRUE;
+  recorder->save_to_file = save_to_file;
   gtk_inspector_recorder_add_recording (recorder, recorder->recording);
 }
 
@@ -2752,14 +2786,23 @@ gtk_inspector_recorder_record_render (GtkInspectorRecorder *recorder,
 
   if (recorder->stop_after_next_frame)
     {
-      GtkSingleSelection *selection;
+      GdkClipboard *clipboard;
 
       recorder->stop_after_next_frame = FALSE;
       gtk_inspector_recorder_set_recording (recorder, FALSE);
 
-      selection = GTK_SINGLE_SELECTION (gtk_list_view_get_model (GTK_LIST_VIEW (recorder->recordings_list)));
-      gtk_single_selection_set_selected (selection, g_list_model_get_n_items (G_LIST_MODEL (selection)) - 1);
-      render_node_clip (NULL, recorder);
+      clipboard = gtk_widget_get_clipboard (GTK_WIDGET (recorder));
+      gdk_clipboard_set (clipboard, GSK_TYPE_RENDER_NODE, node);
+
+      if (recorder->save_to_file)
+        {
+          char *filename;
+          static int counter = 0;
+
+          filename = g_strdup_printf ("gtk-frame%d.node", counter++);
+          gsk_render_node_write_to_file (node, filename, NULL);
+          g_free (filename);
+        }
     }
 }
 
